#!/usr/bin/env python
# Description = Predict subcellular localization based on PsortB

import os
import subprocess
from collections import defaultdict


class PsortbSubcellularLocalization(object):
    def __init__(self, selected_proteins, psortb_path,
                 fasta_path, outpath):
        self._selected_proteins = selected_proteins
        self._psortb_path = psortb_path
        self._fasta_path = fasta_path
        self._outpath = outpath
        
        self._selected_protein_set = set()
        self._localization_dict = defaultdict(
            lambda: defaultdict(float))
        
    def streamline_psortb_localization_analysis(self):
        '''To call from apricot'''
        self.parse_selected_data()
        self.run_psortb_analysis()
        self.parse_psort_outfiles()
        self.store_compiled_data()
        self.create_job_completion_file()
        
    def parse_selected_data(self):
        '''Parses selected data for Uids'''
        with open(self._selected_proteins, 'r') as in_fh:
            for entry in in_fh:
                if not entry.startswith('Entry'):
                    self._selected_protein_set.add(entry.split('\t')[0])
        return self._selected_protein_set
    
    def run_psortb_analysis(self):
        '''Runs PsortB for identifying subcellular
        localization information of the query proteins'''
        for files in os.listdir(self._fasta_path):
            if files.split('.')[0] in self._selected_protein_set:
                print("Psortb subcellular localization analysis for %s" % files)
                subprocess.Popen(
                ["perl %s -n %s/%s > %s/%s_localization.csv" %
                 (self._psortb_path, self._fasta_path,
                  files, self._outpath, files.split('.')[0])], shell=True).wait()
                
    def parse_psort_outfiles(self):
        '''Parses PsortB output files for compiling information'''
        for files in os.listdir(self._outpath):
            protein = files.split('_')[0]
            with open(self._outpath+'/'+files, 'r') as in_fh:
                for result in in_fh.read().split('Final'):
                    if "Prediction:" in result:
                        prediction = result.split('\n')[1].replace(' ', '')
                        #print(prediction)
                        if 'Cytoplasmic' in prediction:
                            if not 'Membrane' in prediction:
                                localization = 'Cytoplasmic'
                                score = prediction.split('Cytoplasmic')[1]
                            else:
                                localization = 'Cytoplasmic-Membrane'
                                score = prediction.split('CytoplasmicMembrane')[1]
                        elif 'Periplasmic' in prediction:
                            localization = 'Periplasmic'
                            score = prediction.split('Periplasmic')[1]
                        elif 'Extracellular' in prediction:
                            localization = 'Extracellular'
                            score = prediction.split('Extracellular')[1]
                        elif 'OuterMembrane' in prediction:
                            localization = 'OuterMembrane'
                            score = prediction.split('OuterMembrane')[1]
                        elif 'Unknown' in prediction:
                            localization = 'Unknown'
                            score = '-'
                        else:
                            print("this entry's localization is not listed: %s" % prediction)
                        self._localization_dict[localization][protein] = score
                        
        return self._localization_dict
    
    def store_compiled_data(self):
        '''Creates output file with PsortB derived information'''
        outfile = self._outpath+'/psortb_data_summary.csv'
        with open(outfile, 'w') as out_fh:
            out_fh.write("localization prediction by PSORTB:\nProteins\tLocalization\tScore\n")
            for entry in self._localization_dict.items():
                localization = entry[0]
                out_fh.write("Localization: %s\tscore\n" % localization)
                print(localization, len(self._localization_dict[localization]))
                for protein in entry[1].keys():
                    score = self._localization_dict[localization][protein]
                    out_fh.write('%s\t%s\t%s\n' % (protein, localization, score))
        print("Data save is %s.\nPlease check %s for the summary." % (
            self._outpath, outfile))
    
    def create_job_completion_file(self):
        with open(self._outpath+'/psortb_analysis.txt', 'w') as out_fh:
            out_fh.write("Localization for the selected proteins are predicted by PsortB.\n")
            out_fh.write("The files generated by the analysis:.\n")
            out_fh.write('\n'.join(os.listdir(self._outpath)))




#!/usr/bin/env python
# Description = Collect protein ids from UniProt database for the query genes

import os
import csv
import re
from itertools import islice


class CollectUniprotInformation(object):
    '''collection of protein ids for the query genes'''
    def __init__(self, query_path,
                 uniprot_reference_file,
                 mapped_data_path,
                 statistics_path):
        self._query_path = query_path
        self._uniprot_reference_file = uniprot_reference_file
        self._mapped_data_path = mapped_data_path
        self._statistics_path = statistics_path
        
        self._query_protein_set = set()
        self._protein_data_dict = {}
        self._gene_to_protein_dict = {}
        self._protein_protein_available = set()
        
    def read_protein_file(self):
        '''read protein files and collect proteins in a list'''
        for query_file in os.listdir(self._query_path):
            with open(self._query_path+'/'+query_file,
                      'r') as query_protein_fh:
                for each_query in query_protein_fh:
                    if not each_query.strip() == '':
                        self._query_protein_set.add(
                            each_query.strip())
        return self._query_protein_set

    def parse_uniprot_reference_file(self):
        '''parse uniprot reference table for posible protein proteins'''
        with open(self._uniprot_reference_file,
                  'r') as uniprot_reference_fh:
            for protein_entry in csv.reader(
                islice(uniprot_reference_fh, 1,
                       None), delimiter='\t'):
                self._protein_data_dict[protein_entry[0]] = (
                    protein_entry[4], protein_entry[3], protein_entry[1])
        return self._protein_data_dict

    def map_proteins_to_uniprot(self):
        '''Maps query gene ids to the UniProt entry'''
        for each_query in list(self._query_protein_set):
            print(each_query)
            print('Retrieving protein for: %s' % each_query)
            self._gene_to_protein_dict.setdefault(
                each_query, []).append(each_query)
            for protein, description in self._protein_data_dict.items():
                exact_protein_match = re.findall('\\b%s\\b' % each_query.lower(
                    ), str(','.join(description).lower()))
                if len(exact_protein_match) > 0:
                    self._protein_protein_available.add(each_query)
                    self._gene_to_protein_dict.setdefault(each_query,
                                                          []).append(protein)
                    print(protein)
        return self._gene_to_protein_dict, self._protein_protein_available
    
    def create_mapped_file(self):
        '''Creates a file with mapping informtion of the
        query genes to UniProt protein ids'''
        with open(self._mapped_data_path,
                  'w') as gene_to_protein:
            with open(self._statistics_path,
                      'w') as statistics_file:
                for mapped_protein_data in self._gene_to_protein_dict:
                    gene_to_protein.write('%s\n' % ';'.join(
                        self._gene_to_protein_dict.get(mapped_protein_data)))
                statistics_file.write(
                    'Total number of proteins: %s\n' % len(
                        list(self._query_protein_set)))
                statistics_file.write(
                    'Total number of available protein proteins: %s\n' % len(
                        list(self._protein_protein_available)))
                uniq_not_available_data = (
                    self._query_protein_set-self._protein_protein_available)
                statistics_file.write(
                    'Total number of unavailable protein proteins: %s\n' %
                    len(uniq_not_available_data))
                statistics_file.write('\nAvailable protein proteins:\n')
                for available in list(self._protein_protein_available):
                    statistics_file.write('%s\n' % available)
                statistics_file.write('\nUnavailable protein proteins: \n')
                for unavailable in uniq_not_available_data:
                    statistics_file.write('%s\n' % unavailable)
                print('Proteins sequences are generated for '
                      'truly mapped genes.')
                statistics_file.close()
                print('Proteins are mapped to the available proteins.')



#!/usr/bin/env python
# Description = Creates an overview for APRICOT analysis

import os
from collections import defaultdict


class CreateAnalysisSummary(object):
    '''APRICOT analysis summary'''
    
    def __init__(self, query_map, selected_domains,
                 unfilter_path, outpath):
        self._query_map = query_map
        self._selected_domains = selected_domains
        self._unfilter_path = unfilter_path
        self._outpath = outpath
        
        self._query_info = {}
        self._qury_info_list = []
        self._domain_keyword_info = {}
        self._prot_domain_dict = defaultdict(lambda: defaultdict(
            lambda: defaultdict(lambda: set())))
        self._info_tag_dict = {}
        self._prot_go_dict = {}
        
    def streamline_create_analysis_summary(self):
        '''To call from apricot exe'''
        self.count_queries()
        self.count_domains()
        self.domain_stat()
        self.record_selected_stat()
        
    def count_queries(self):
        '''Records query mapping information'''
        with open(self._query_map, 'r') as in_fh:
            self._query_info.setdefault('mapped', set()).add('')
            self._query_info.setdefault('unmapped', set()).add('')
            for entry in in_fh:
                if not entry.strip() == '':
                    if ':' not in entry.strip():
                        self._query_info.setdefault(
                            'mapped', set()).add(entry.strip())
                    else:
                        if not entry.strip().split(':')[1] == 'unmapped':
                            self._query_info.setdefault(
                                'mapped', set()).add(entry.split(':')[0])
                        else:
                            self._query_info.setdefault(
                                'unmapped', set()).add(entry.split(':')[0])
                        self._qury_info_list.append(
                            entry.strip().replace(':', '\t'))
        return self._query_info, self._qury_info_list
    
    def count_domains(self):
        '''Records selected domain information'''
        with open(self._selected_domains, 'r') as in_fh:
            for entry in in_fh:
                if not entry.strip() == '' and not entry.startswith(
                        'ReferenceId'):
                    keyword = entry.strip().split("\t")[-1]
                    self._domain_keyword_info.setdefault(
                        keyword, []).append(entry.strip())
        return self._domain_keyword_info
        
    def domain_stat(self):
        '''Records statistics of predicted domains'''
        for files in os.listdir(self._unfilter_path):
            if '_unfiltered_all_prediction.csv' in files:
                if 'cdd' in files.lower():
                    data_source = 'CDD'
                else:
                    data_source = 'InterPro'
                with open(self._unfilter_path+'/'+files, 'r') as in_fh:
                    for entry in in_fh:
                        if not entry.startswith('Resource'):
                            if not entry.strip() == '':
                                data_detail = NonFilteredData(
                                    entry.strip().split('\t'))
                                self._prot_domain_dict[
                                    data_source][data_detail.uid][
                                    data_detail.domain_id].add("%s:%s" % (
                                        data_detail.start,
                                        data_detail.stop))
                                self._info_tag_dict["%s:%s:%s:%s:%s" % (
                                    data_source, data_detail.uid,
                                    data_detail.domain_id,
                                    data_detail.start, data_detail.stop)] = "%s:%s" % (
                                    data_detail.parameter, data_detail.domain_tag)
            if not 'cdd_unfiltered_all_prediction.csv' in os.listdir(
                    self._unfilter_path):
                print("The folder %s does not contain cdd_unfiltered_all_prediction.csv." %
                      self._inpath+'/output/1_all_domain_predictions')
            if not 'ipr_unfiltered_all_prediction.csv' in os.listdir(
                    self._unfilter_path):
                print("The folder %s does not contain ipr_unfiltered_all_prediction.csv." %
                      self._inpath+'/output/1_all_domain_predictions')
        return self._prot_domain_dict, self._info_tag_dict
    
    def record_selected_stat(self):
        '''Creates an output file with summary information'''
        with open(self._outpath, 'w') as out_fh:
            out_fh.write("Total query proteins: %s\n" % str(
                len(list(self._query_info[
                    'mapped'])+list(self._query_info['unmapped']))-2))
            out_fh.write(
                "Total query proteins mapped to UniProt IDs: %s\n" % str(
                    len(self._query_info['mapped'])-1))
            out_fh.write(
                "Total query proteins not mapped to UniProt IDs: %s\n\n" % str(
                    len(self._query_info['unmapped'])-1))
            out_fh.write("ID mapping details:\n%s\n\n" % '\n'.join(
                self._qury_info_list))
            out_fh.write(
                "Keywords used for retreiving the domains of interest: %s\n"
                % len(self._domain_keyword_info.keys()))
            for kw in self._domain_keyword_info.keys():
                out_fh.write("%s\t%s\n" % (
                    kw, len(self._domain_keyword_info[kw])))
            for database in self._prot_domain_dict.keys():
                out_fh.write("\n--------------------------------------------------------")
                out_fh.write("--------------------------------------------------------\n\n")
                out_fh.write("Domain prediction summary for %s:\n" % database)
                total_proteins = len(
                    list(set(self._prot_domain_dict[database].keys())))
                out_fh.write(
                    "Total proteins with domains: %s\n" % total_proteins)
                protein_pred_dict = {}
                parameter_selected_data = {}
                domain_selected_data = {}
                proteins_location = {}
                for proteins in self._prot_domain_dict[database].keys():
                    total_locations = 0
                    protein_pred_dict[proteins] = len(list(set(
                        self._prot_domain_dict[
                            database][proteins].keys())))
                    for domain_id in self._prot_domain_dict[
                            database][proteins].keys():
                        total_locations += len(self._prot_domain_dict[
                            database][proteins][domain_id])
                        for location in self._prot_domain_dict[
                                database][proteins][domain_id]:
                            tag_key = "%s:%s:%s:%s" % (
                                database, proteins, domain_id, location)
                            if self._info_tag_dict[tag_key].split(
                                    ':')[0] == 'ParameterSelected':
                                parameter_selected_data.setdefault(
                                    proteins, set()).add(domain_id)
                                if self._info_tag_dict[tag_key].split(
                                        ':')[1] == 'Selected':
                                    domain_selected_data.setdefault(
                                        proteins, set()).add(domain_id)
                    proteins_location[proteins] = total_locations
                out_fh.write(
                    "Query\tTotal predictions (locations identified)\tUnique "
                    "domains (ids)\tUnique domains that pass the parameter "
                    "filter\n")
                preds = 0
                uniq = 0
                param_uniq = 0
                for uid in parameter_selected_data.keys():
                    out_fh.write("%s\t%s\t%s\t%s\n" % (
                        uid, proteins_location[uid],
                        protein_pred_dict[uid],
                        len(parameter_selected_data[uid])))
                    preds += proteins_location[uid]
                    uniq += protein_pred_dict[uid]
                    param_uniq += len(parameter_selected_data[uid])
                out_fh.write(
                    "Total domain predictions (locations in the queries): %s\n"
                    % str(preds))
                out_fh.write(
                    "Unique domains predicted in the queries: %s\n" % str(
                        uniq))
                out_fh.write(
                    "Unique domain that pass the parameter filter: %s\n" %
                    str(param_uniq))
                out_fh.write(
                    "\nTotal proteins with 'domains of interest' that pass "
                    "the parameter filter:\n%s\n" %
                    ', '.join(domain_selected_data.keys()))
                out_fh.write("Proteins\tDomains of interest\tDomain count\n")
                for each_key in domain_selected_data.keys():
                    out_fh.write("%s\t%s\t%s\n" % (
                        each_key, ', '.join(domain_selected_data[each_key]),
                        len(domain_selected_data[each_key])))
 

class NonFilteredData(object):
    '''all the column details of unfiltered table'''
    def __init__(self, row):
        self.resource = row[0]
        self.uid = row[1]
        self.resource_id = row[2]
        self.domain_id = row[3]
        self.short_name = row[4]
        self.full_name = row[5]
        self.keyword = row[6]
        self.domain_go = row[7]
        self.members = row[8]
        self.domain_length = row[9]
        self.start = row[10]
        self.stop = row[11]
        self.stats = row[12:-2]
        self.domain_tag = row[-1]
        self.parameter = row[-2]



#!/usr/bin/env python
# Description = Folder creation for different analysis modules in APRICOT pipeline

import os


def setup_analysis_folders(analysis_path):
    '''sets up folder for APRICOT analysis,
    default folder name is APRICOT_analysis'''
    create_main_folders(analysis_path)
    create_domain_analysis_path(analysis_path)
    create_subanalysis_path(analysis_path)


def create_main_folders(analysis_path):
    '''creates main folders in the APRICOT analysis folder'''
    if not os.path.exists(analysis_path):
        os.mkdir(analysis_path)
    if not os.path.exists('source_files'):
        os.mkdir('source_files')
    if not os.path.exists(analysis_path+'/input'):
        os.mkdir(analysis_path+'/input')
    if not os.path.exists(analysis_path+'/output'):
        os.mkdir(analysis_path+'/output')


def create_domain_analysis_path(analysis_path):
    '''sets up domain analysis and output folders'''
    if not os.path.exists('source_files/domain_data'):
        os.mkdir('source_files/domain_data')
    if not os.path.exists('source_files/reference_db_files'):
        os.mkdir('source_files/reference_db_files')
    if not os.path.exists(
            'source_files/domain_data/cdd'):
        os.mkdir(
                'source_files/domain_data/cdd')
    if not os.path.exists(
            'source_files/domain_data/interpro'):
        os.mkdir(
                'source_files/domain_data/interpro')


def create_subanalysis_path(analysis_path):
    '''create sub paths in the analysis folders'''
    for paths in ('input/query_proteins',
                  'input/mapped_query_annotation',
                  'input/uniprot_reference_table',
                  'output/0_predicted_domains',
                  'output/1_compiled_domain_information',
                  'output/2_selected_domain_information',
                  'output/3_annotation_scoring',
                  'output/4_additional_annotations',
                  'output/5_analysis_summary',
                  'output/format_output_data',
                  'output/visualization_files'):
        if not os.path.exists(analysis_path+'/'+paths):
            os.mkdir(analysis_path+'/'+paths)
    for sub_analysis_path in (
        'input/mapped_query_annotation/xml_path_mapped_query',
        'input/mapped_query_annotation/fasta_path_mapped_query',
        'input/mapped_query_annotation/mapped_protein_xml_info_tables',
        'output/0_predicted_domains/cdd_analysis',
        'output/0_predicted_domains/ipr_analysis',
        'output/1_compiled_domain_information/selected_data',
        'output/1_compiled_domain_information/unfiltered_data',
        'output/2_selected_domain_information/classified_data',
        'output/2_selected_domain_information/combined_data',
        'output/4_additional_annotations/protein_localization',
        'output/4_additional_annotations/protein_secondary_structure',
        'output/4_additional_annotations/pdb_sequence_prediction',
        'output/format_output_data/excel_files',
        'output/format_output_data/html_files',
        'output/visualization_files/overview_and_statistics',
        'output/visualization_files/secondary_structure',
        'output/visualization_files/subcellular_localization',
        'output/visualization_files/homologous_pdb_msa',
            'output/visualization_files/domain_highlighting'):
        if not os.path.exists(analysis_path+'/'+sub_analysis_path):
            os.mkdir(analysis_path+'/'+sub_analysis_path)
            



#!/usr/bin/env python
# Description = Stores user provided keywords in files.


def keyword_domains_files(kw_file_path, keywords):
    with open(kw_file_path+'/keywords_for_domain_selection.txt',
              'w') as out_fh:
        if ',' in keywords:
            for each_kw in keywords.split(','):
                if ' ' in each_kw:
                    each_kw.replace(' ', '-')
                out_fh.write('%s\n' % each_kw)
        else:
            out_fh.write(keywords)
            print("------\nSingle keyword for domain selection "
                  "has been given.")
            print("Multiple keywords can be provided as comma separated "
                  "list.\n------")


def keyword_class_files(kw_file_path, keywords):
    with open(kw_file_path+'/keywords_for_result_classification.txt',
              'w') as out_fh:
        if ',' in keywords:
            out_fh.write('\n'.join(keywords.split(',')))
        else:
            out_fh.write(keywords)
            print("------\nSingle keyword for result "
                  "classification has been given.")
            print("Multiple keywords can be provided as "
                  "comma separated list.\n------")



#!/usr/bin/env python
# Description = selects rna related domains from cdd database

import re


class RnaRelatedCDDSelection(object):
    def __init__(self, keywords_file,
                 cdd_whole_data_file,
                 interpro_mapped_cdd,
                 domain_data_path):
        self._keywords_file = keywords_file
        self._cdd_whole_data_file = cdd_whole_data_file
        self._interpro_mapped_cdd = interpro_mapped_cdd
        self._domain_data_path = domain_data_path
        
        self.cdd_whole_data_list = []
        self._cdd_dict = {}
        self._mapped_cdd_members = {}
    
    def select_cdd_domains(self):
        '''To call from apriot'''
        self.read_keyword_file()
        self.read_interpro_mapped_cdd_file()
        self.read_cdd_whole_data_file()
        self.create_keyword_selected_domain_file()
        
    def read_keyword_file(self):
        '''reads user provided keywords for domain selection'''
        self._keyword_list = [rna_keyword.strip()
                              for rna_keyword in open(
                                self._keywords_file, 'r')]
        return self._keyword_list
    
    def read_interpro_mapped_cdd_file(self):
        '''Parses cdd interpro mapped file for
        the extraction of common information'''
        with open(self._interpro_mapped_cdd, 'r') as in_fh:
            for entry in in_fh:
                ipr_id = entry.strip().split('\t')[0]
                ipr_members = entry.strip().split('\t')[1]
                cdd_id = entry.strip().split('\t')[2]
                cdd_member = entry.strip().split('\t')[3]
                domain_length = entry.strip().split('\t')[4]
                self._mapped_cdd_members[cdd_member] = ipr_members
        self._mapped_cdd_members
    
    def read_cdd_whole_data_file(self):
        '''Parses CDD annotation data from table'''
        with open(self._cdd_whole_data_file, 'r') as cdd_data_fh:
            for cdd_entry in cdd_data_fh:
                if 'smart' in cdd_entry.split('\t')[1]:
                    cdd_entry = cdd_entry.replace(
                        cdd_entry.split('\t')[1], 'SM'+cdd_entry.split(
                            '\t')[1].split('smart')[-1])
                elif 'pfam' in cdd_entry.split('\t')[1]:
                    cdd_entry = cdd_entry.replace(
                        cdd_entry.split('\t')[1], 'PF'+cdd_entry.split(
                            '\t')[1].split('pfam')[-1])
                cdd_domain = cdd_entry.split('\t')[1]
                if cdd_domain in self._mapped_cdd_members.keys():
                    members = self._mapped_cdd_members[cdd_domain]
                else:
                    members = 'NA'
                self.cdd_whole_data_list.append(
                    cdd_entry.strip())
                self._cdd_dict[cdd_entry.strip()] = " ".join(cdd_entry.strip(
                    ).split('\t')[2:4])
        return self.cdd_whole_data_list, self._cdd_dict
    
    def create_keyword_selected_domain_file(self):
        '''Creates a file with CDD derived domain of interest'''
        self._keyword_annotation_dict = {}
        self._keyword_selected_domain = {}
        for cdd_entry in self.cdd_whole_data_list:
            domain_full_name = list(self._cdd_dict[
                cdd_entry].lower().replace('-', ' ').replace('_', ' ').split(' '))
            for keyword in self._keyword_list:
                check_keyword = keyword.lower()
                #if 'rna' in check_keyword:
                if '-' in check_keyword:
                    ref_kw_list = check_keyword.split('-')
                    new_kw_list = []
                    for domain_name in domain_full_name:
                        for string in ref_kw_list:
                            if string == 'rna':
                                if re.findall(
                                        r"(?:\s|^)%s(?=\s|$)" % '*rna', domain_name):
                                    if not string in set(new_kw_list):
                                        new_kw_list.append(string)
                            else:
                                if re.findall(
                                        r"(?:\s|^)%s(?=\s|$)" % string, domain_name):
                                    if not string in set(new_kw_list):
                                        new_kw_list.append(string)
                    if str(ref_kw_list) == str(new_kw_list):
                        self._keyword_annotation_dict.setdefault(
                            keyword, []).append(cdd_entry)
                else:
                    for domain_name in domain_full_name:
                        if re.findall(
                                r"(?:\s|^)%s(?=\s|$)" % check_keyword, domain_name):
                            self._keyword_annotation_dict.setdefault(
                                keyword, []).append(cdd_entry)
        for fkeyword in self._keyword_list:
            fkeyword = fkeyword.replace(' ', '_')
            with open(self._domain_data_path + '/' + fkeyword +
                      '_related_cdd_ids.tab',
                      'w') as keyword_specific_domain:
                if self._keyword_annotation_dict.get(fkeyword):
                    for each_entry in self._keyword_annotation_dict[fkeyword]:
                        each_entry = each_entry.replace(
                            each_entry.split('\t')[3], " ".join(
                                each_entry.split(
                                    '\t')[3].split())).replace(';', ',')
                        cdd_domain = each_entry.split('\t')[1]
                        if cdd_domain in set(self._mapped_cdd_members.keys()):
                            members = self._mapped_cdd_members[cdd_domain]
                        else:
                            members = 'NA'
                        self._keyword_selected_domain.setdefault(
                            '%s\t%s\t%s' % (
                                '\t'.join(each_entry.split('\t')[0:-1]),
                                members,
                                each_entry.split(
                                    '\t')[-1]), set()).add(fkeyword)
                        keyword_specific_domain.write('%s\t%s\t%s\t%s\n' % (
                            '\t'.join(each_entry.split('\t')[0:-1]), members,
                            each_entry.split('\t')[-1], fkeyword))
        with open(self._domain_data_path+'/all_keyword_selected_cdd_data.tab',
                  'w') as keyword_selected_domain_file:
            for domain_entry in self._keyword_selected_domain.keys():
                keywords = ', '.join(list(
                    self._keyword_selected_domain[domain_entry]))
                keyword_selected_domain_file.write(
                    '%s\t%s\n' % (domain_entry, keywords))



#!/usr/bin/env python
# Description = Annotates selected queries with proteins and domain information

import os


class SelectedProteinTable(object):
    
    def __init__(self, filtered_data_path,
                 uniprot_reference_table,
                 xml_feature_table,
                 selected_data_table):
        self._filtered_data_path = filtered_data_path
        self._uniprot_reference_table = uniprot_reference_table
        self._xml_feature_table = xml_feature_table
        self._selected_data_table = selected_data_table
        self._filter_data_header = ''
        self._reference_data_header = ''
        self._feature_data_header = ''
        self._filter_data_dict = {}
        self._up_ref_dict = {}
        self._protein_feature_dict = {}
        
    def streamline_selected_protein_table(self):
        '''To call from apricot exe'''
        self.parse_filtered_data()
        self.parse_uniprot_reference()
        self.read_xml_protein_table()
        self.create_selected_data_table()
        
    def parse_filtered_data(self):
        '''Parse filtered information'''
        for files in os.listdir(self._filtered_data_path):
            if 'id' not in files:
                with open(self._filtered_data_path+'/'+files,
                          'r') as in_fh:
                    for entry in in_fh:
                        if entry.startswith('Resource'):
                            self._filter_data_header = entry.strip()
                        else:
                            uid = entry.split('\t')[1]
                            self._filter_data_dict.setdefault(
                                uid, set()).add(entry.strip())
        return self._filter_data_header, self._filter_data_dict
        
    def parse_uniprot_reference(self):
        '''Parse UniProt reference table'''
        for files in os.listdir(self._uniprot_reference_table):
            if '_reference' in files:
                with open(self._uniprot_reference_table+'/'+files,
                          'r') as in_fh:
                    for entry in in_fh:
                        if entry.startswith('Entry'):
                            self._reference_data_header = entry.strip()
                        else:
                            uid = entry.split('\t')[0]
                            if uid in set(
                                    self._filter_data_dict.keys()):
                                self._up_ref_dict[uid] = entry.strip()
        return self._reference_data_header, self._up_ref_dict
    
    def read_xml_protein_table(self):
        '''Read and record data from xml'''
        with open(self._xml_feature_table
                  ) as protein_xml_table_fh:
            for entry in protein_xml_table_fh:
                if entry.startswith('Entry'):
                    self._feature_data_header = entry.strip()
                else:
                    uid = entry.split('\t')[0]
                    if uid in set(
                            self._filter_data_dict.keys()):
                        self._protein_feature_dict[uid] = entry.strip()
        return self._feature_data_header, self._protein_feature_dict

    def create_selected_data_table(self):
        '''Creates selected data table'''
        with open(self._selected_data_table, 'w') as out_fh:
            filter_header = FilteredData(
                self._filter_data_header.split('\t'))
            reference_header = UniProtReference(
                self._reference_data_header.split('\t'))
            feature_header = ProteinFeature(
                self._feature_data_header.split('\t'))
            self._entries_into_the_file(
                out_fh, reference_header, feature_header, filter_header)
            for uid in self._filter_data_dict.keys():
                reference_data = UniProtReference(
                    self._up_ref_dict[uid].split('\t'))
                feature_data = ProteinFeature(
                    self._protein_feature_dict[uid].split('\t'))
                if len(self._filter_data_dict[uid]) > 1:
                    for entry in self._filter_data_dict[uid]:
                        filter_data = FilteredData(entry.split('\t'))
                        if filter_data.parameter == 'ParameterSelected':
                            self._entries_into_the_file(
                                out_fh, reference_data, feature_data,
                                filter_data)
                else:
                    filter_data = FilteredData(
                        list(self._filter_data_dict[uid])[0].split('\t'))
                    if filter_data.parameter == 'ParameterSelected':
                        self._entries_into_the_file(
                            out_fh, reference_data, feature_data, filter_data)
                    
    def _entries_into_the_file(self, out_fh,
                               reference_data, feature_data, filter_data):
        '''Creates an output with the protein features'''
        out_fh.write("\t".join([
            reference_data.uid,
            reference_data.entry_name, reference_data.protein_name,
            reference_data.species, reference_data.length,
            reference_data.gene_name, feature_data.gene_locus,
            feature_data.type, feature_data.go,
            feature_data.embl, feature_data.pdb, feature_data.kegg,
            feature_data.interpro, feature_data.pfam,
            feature_data.pubmed, filter_data.resource,
            filter_data.resource_id, filter_data.domain_id,
            filter_data.short_name, filter_data.full_name,
            filter_data.keyword, filter_data.domain_go, filter_data.members,
            filter_data.domain_length, filter_data.start,
            filter_data.stop, '\t'.join(filter_data.stats)])+'\n')


class ProteinFeature(object):
    
    def __init__(self, row):
        self.uid = row[0]
        self.protein_name = row[1]
        self.gene = row[2]
        self.gene_locus = row[3]
        self.species = row[4]
        self.pubmed = row[5]
        self.embl = row[6]
        self.refseq = row[7]
        self.kegg = row[8]
        self.pdb = row[9]
        self.go = row[10]
        self.interpro = row[11]
        self.pfam = row[12]
        self.type = row[13]


class UniProtReference(object):
    
    def __init__(self, row):
        self.uid = row[0]
        self.entry_name = row[1]
        self.status = row[2]
        self.protein_name = row[3]
        self.gene_name = row[4]
        self.species = row[5]
        self.length = row[6]
        try:
            self.gene_ontology = row[7]
        except:
            self.gene_ontology = '-'


class FilteredData(object):
    
    def __init__(self, row):
        self.resource = row[0]
        self.uid = row[1]
        self.resource_id = row[2]
        self.domain_id = row[3]
        self.short_name = row[4]
        self.full_name = row[5]
        self.keyword = row[6]
        self.domain_go = row[7]
        self.members = row[8]
        self.domain_length = row[9]
        self.start = row[10]
        self.stop = row[11]
        self.stats = row[12:]
        self.parameter = row[-1]



#!/usr/bin/env python
# Description = Formt PDB secodary structure database

import argparse


parser = argparse.ArgumentParser()
parser.add_argument(
    "-i", "--pdb_ss_file", help="input secondary structure")
parser.add_argument(
    "-seq", "--pdb_seq_path", help="output pdb sequence")
parser.add_argument(
    "-str", "--pdb_str_path", help="output pdb structure")
args = parser.parse_args()


"Sorts secondary structre and sequence from PDB"
sequence_id_dict = {}
out_seq_file = args.pdb_seq_path+'/pdb_sequence.txt'
out_ss_file = args.pdb_str_path+'/pdb_secstr.txt'
    
with open(args.pdb_ss_file) as in_fh:
    with open(out_seq_file, 'w') as out_seq_fh:
        with open(out_ss_file, 'w') as out_ss_fh:
            for data in in_fh.read().split('>'):
                if "sequence" in data:
                    seq_id = data.split(':')[0]
                    sequence_id_dict[seq_id] = data
                    out_seq_fh.write('>%s' % data)
                elif "secstr" in data:
                    str_id = data.split(':')[0]
                    data = data.replace(' ', '0')
                    seq_len = (len(sequence_id_dict[str_id]) - len(data))
                    for short in range(int(seq_len)):
                        data = data.strip()+'0'
                        out_ss_fh.write('>%s' % data)



#!/usr/bin/env python
# Description = selects domains from cdd database

import re


class RnaRelatedDomainSelection(object):
    '''classification of data'''
    def __init__(self, keywords_file,
                 cdd_whole_data_file,
                 interpro_mapped_cdd,
                 domain_data_path):
        self._keywords_file = keywords_file
        self._cdd_whole_data_file = cdd_whole_data_file
        self._interpro_mapped_cdd = interpro_mapped_cdd
        self._domain_data_path = domain_data_path
        
        self.cdd_whole_data_list = []
        self._cdd_dict = {}
        self._mapped_cdd_members = {}
        
    def read_keyword_file(self):
        '''read keywords for selection'''
        self._keyword_list = [rna_keyword.strip()
                              for rna_keyword in open(
                                self._keywords_file, 'r')]
        return self._keyword_list
    
    def read_interpro_mapped_cdd_file(self):
        ''''''
        with open(self._interpro_mapped_cdd, 'r') as in_fh:
            for entry in in_fh:
                ipr_members = entry.strip().split('\t')[1]
                cdd_member = entry.strip().split('\t')[3]
                self._mapped_cdd_members[cdd_member] = ipr_members
        self._mapped_cdd_members
    
    def read_cdd_whole_data_file(self):
        '''read and annotation data from table'''
        with open(self._cdd_whole_data_file, 'r') as cdd_data_fh:
            for cdd_entry in cdd_data_fh:
                if 'smart' in cdd_entry.split('\t')[1]:
                    cdd_entry = cdd_entry.replace(
                        cdd_entry.split('\t')[1], 'SM'+cdd_entry.split(
                            '\t')[1].split('smart')[-1])
                elif 'pfam' in cdd_entry.split('\t')[1]:
                    cdd_entry = cdd_entry.replace(
                        cdd_entry.split('\t')[1], 'PF'+cdd_entry.split(
                            '\t')[1].split('pfam')[-1])
                cdd_domain = cdd_entry.split('\t')[1]
                if cdd_domain in self._mapped_cdd_members.keys():
                    members = self._mapped_cdd_members[cdd_domain]
                else:
                    members = 'NA'
                self.cdd_whole_data_list.append(
                    cdd_entry.strip())
                self._cdd_dict[cdd_entry.strip()] = cdd_entry.strip(
                    ).split('\t')[3]
        return self.cdd_whole_data_list, self._cdd_dict
    
    def create_keyword_selected_domain_file(self):
        '''create rne related domain list'''
        self._keyword_annotation_dict = {}
        self._keyword_selected_domain = []
        for cdd_entry in self.cdd_whole_data_list:
            for keyword in self._keyword_list:
                if ' ' in keyword:
                    key_list = []
                    for each_key in keyword.split(' '):
                        key_list.append(each_key)
                    match = re.search(r'\b%s*|\W%s\b'%(key_list[0].lower(),
                                                       key_list[1].lower()),
                                      self._cdd_dict[cdd_entry])
                    if match:
                        self._keyword_annotation_dict.setdefault(
                            keyword, []).append(cdd_entry)
                else:
                    match = re.search(r'\b%s\b'%keyword, self._cdd_dict[cdd_entry])
                    if match:
                        self._keyword_annotation_dict.setdefault(
                            keyword, []).append(cdd_entry)
                        
        for fkeyword in self._keyword_list:
            fkeyword = fkeyword.replace(' ', '_')
            with open(self._domain_data_path+'/'+fkeyword+'_related_cdd_ids.tab',
                      'w') as keyword_specific_domain:
                if self._keyword_annotation_dict.get(fkeyword):
                    for each_entry in self._keyword_annotation_dict[fkeyword]:
                        each_entry = each_entry.replace(
                            each_entry.split('\t')[3], " ".join(
                                each_entry.split('\t')[3].split())).replace(';', ',')
                        cdd_domain = each_entry.split('\t')[1]
                        if cdd_domain in set(self._mapped_cdd_members.keys()):
                            members = self._mapped_cdd_members[cdd_domain]
                        else:
                            members = 'NA'
                        keyword_specific_domain.write('%s\t%s\t%s\n'%(
                            '\t'.join(each_entry.split('\t')[0:-1]), members,
                            each_entry.split('\t')[-1]))
                        self._keyword_selected_domain.append(
                            ('%s\t%s\t%s' % (
                                '\t'.join(each_entry.split('\t')[0:-1]),
                                members, each_entry.split('\t')[-1])))
            
        uniq_keyword_selected_domains = list(
            set(self._keyword_selected_domain))
        with open(self._domain_data_path+'/all_keyword_selected_cdd_data.tab',
                  'w') as keyword_selected_domain_file:
            for domain_entry in uniq_keyword_selected_domains:
                keyword_selected_domain_file.write(
                    '%s\n' % str(domain_entry))



#!/usr/bin/env python
# Description = Annotates selected queries with available information

import os


class SelectedProteinTableWithoutUniprot(object):
    def __init__(self, filtered_data_path,
                 selected_data_table):
        self._filtered_data_path = filtered_data_path
        self._selected_data_table = selected_data_table
        self._filter_data_header = ''
        self._reference_data_header = ''
        self._feature_data_header = ''
        self._filter_data_dict = {}
        self._up_ref_dict = {}
        self._protein_feature_dict = {}
        
    def streamline_selected_protein_table_without_uniprot(self):
        '''To call from apricot exe'''
        self.parse_filtered_data()
        self.create_selected_data_table()
        
    def parse_filtered_data(self):
        '''Parse filtered information'''
        for files in os.listdir(self._filtered_data_path):
            if 'id' not in files:
                with open(self._filtered_data_path+'/'+files,
                          'r') as in_fh:
                    for entry in in_fh:
                        if entry.startswith('Resource'):
                            self._filter_data_header = entry.strip()
                        else:
                            uid = entry.split('\t')[1]
                            self._filter_data_dict.setdefault(
                                uid, set()).add(entry.strip())
        return self._filter_data_header, self._filter_data_dict
    
    def create_selected_data_table(self):
        '''Creates selected data table'''
        with open(self._selected_data_table, 'w') as out_fh:
            filter_header = FilteredData(
                self._filter_data_header.split('\t'))
            self._entries_into_the_file(out_fh, filter_header)
            for uid in self._filter_data_dict.keys():
                if len(self._filter_data_dict[uid]) > 1:
                    for entry in self._filter_data_dict[uid]:
                        filter_data = FilteredData(entry.split('\t'))
                        if filter_data.parameter == 'ParameterSelected':
                            self._entries_into_the_file(out_fh, filter_data)
                else:
                    filter_data = FilteredData(
                        list(self._filter_data_dict[uid])[0].split('\t'))
                    if filter_data.parameter == 'ParameterSelected':
                        self._entries_into_the_file(out_fh, filter_data)
                    
    def _entries_into_the_file(self, out_fh, filter_data):
        '''Creates an output with the protein features'''
        out_fh.write("\t".join([
            filter_data.uid,
            filter_data.resource_id, filter_data.domain_id,
            filter_data.short_name, filter_data.full_name,
            filter_data.keyword, filter_data.domain_go, filter_data.members,
            filter_data.domain_length, filter_data.start,
            filter_data.stop, '\t'.join(filter_data.stats)])+'\n')


class FilteredData(object):
    def __init__(self, row):
        self.resource = row[0]
        self.uid = row[1]
        self.resource_id = row[2]
        self.domain_id = row[3]
        self.short_name = row[4]
        self.full_name = row[5]
        self.keyword = row[6]
        self.domain_go = row[7]
        self.members = row[8]
        self.domain_length = row[9]
        self.start = row[10]
        self.stop = row[11]
        self.stats = row[12:]
        self.parameter = row[-1]



#!/usr/bin/env python
# Description = Identifies similar PDB structures based on secondary structures

import os
import sys

try:
    import subprocess
except ImportError:
    print('Python package subprocess is missing. Please install/update.\n')
    sys.exit(0)
try:
    from Bio.Blast import NCBIXML
except ImportError:
    print('Python package Biopython is missing. Please install/update.\n')
    sys.exit(0)


class PdbHomologyAnalysis(object):
    def __init__(self, selected_proteins, pdb_path,
                 fasta_path, outpath):
        self._selected_proteins = selected_proteins
        self._pdb_path = pdb_path
        self._fasta_path = fasta_path
        self._outpath = outpath
        
        self._selected_protein_set = set()
        
    def streamline_pdb_homology_analysis(self):
        self.parse_selected_data()
        self.run_pdb_analysis()
        self.parse_blast_xml()
        self.create_job_completion_file()
        
    def parse_selected_data(self):
        with open(self._selected_proteins, 'r') as in_fh:
            for entry in in_fh:
                if not entry.startswith('Entry'):
                    self._selected_protein_set.add(entry.split('\t')[0])
        return self._selected_protein_set
    
    def run_pdb_analysis(self):
        for files in os.listdir(self._fasta_path):
            if files.split('.')[0] in self._selected_protein_set:
                print("PDB homology analysis for %s" % files)
                subprocess.Popen(
                    ["bin/reference_db_files/blast/blastp -query %s/%s -db "
                     "%s -task blastp -outfmt 0 -out %s/%s.txt" % (
                        self._fasta_path, files, self._pdb_path,
                        self._outpath, files.split('.')[0])],
                    shell=True).wait()
                subprocess.Popen(
                    ["bin/reference_db_files/blast/blastp -query %s/%s "
                     "-db %s -task blastp -outfmt 5 -out %s/%s.xml" % (
                         self._fasta_path, files, self._pdb_path,
                         self._outpath, files.split('.')[0])],
                    shell=True).wait()
                
    def parse_blast_xml(self):
        '''parese blast xml to get seq'''
        for files in os.listdir(self._outpath):
            if '.xml' in files:
                hit_set = set('None')
                identity_align_dict = {}
                with open(self._outpath+'/'+files) as blast_fh:
                    blast_records = NCBIXML.parse(blast_fh)
                    for blast_record in blast_records:
                        for alignment in blast_record.alignments:
                            pdb_id = alignment.title.split(
                                " ")[1].split(':')[0]
                            sub_id = alignment.title.split(
                                " ")[1].split(':')[1]
                            pdb_header = "%s_%s" % (pdb_id, sub_id)
                            for hsp in alignment.hsps:
                                align_seq = hsp.sbjct
                                hsp_identity = hsp.identities
                                identity_align_dict[float(hsp_identity)
                                ] = ">%s (hsp_identity: %s)\n%s\n" % (
                                    pdb_header, hsp_identity, align_seq)
                with open(self._outpath+'/' + files.split('.')[0] +
                          '_top5.fasta', 'w') as out_fh:
                    with open(
                            self._fasta_path+'/' + files.split(
                                '.')[0] + '.fasta', 'r') as fa_fh:
                        fasta_line = []
                        for entry in fa_fh:
                            if '>' in entry:
                                out_fh.write(">%s\n" % entry.split('|')[1])
                            else:
                                fasta_line.append(entry.strip())
                        out_fh.write(''.join(fasta_line)+'\n')
                    for entry in reversed(sorted(identity_align_dict.keys())):
                        hit_id = identity_align_dict[
                            entry].split('>')[1].split(' ')[0]
                        if hit_id not in hit_set and not len(list(
                                hit_set)) > 6:
                            hit_set.add(hit_id)
                            out_fh.write(identity_align_dict[entry])

    def create_job_completion_file(self):
        with open(self._outpath+'/pdb_analysis.txt', 'w') as out_fh:
            out_fh.write("Structural homologs for the selected "
                         "proteins are calculated from PDB database.\n")
            out_fh.write("The files generated by the analysis:.\n")
            out_fh.write('\n'.join(os.listdir(self._outpath)))



#!/usr/bin/env python
# Description = Classify protein data predicted with domain of interest into subclass

import re


class ProteinClassifier(object):
    '''classification of data'''
    def __init__(self, selected_protein_table,
                 keyword_file,
                 classified_result_path):
        self._selected_protein_table = selected_protein_table
        self._keyword_file = keyword_file
        self._classified_result_path = classified_result_path
        self._protein_data_set = set()
        self._keyword_set = set()
        self._keyword_candidate_data = {}
        self._file_header = ''
        self._classified_data = set()
        self._unclassified_data = set()
        
    def streamline_protein_classification(self):
        self.parse_protein_table()
        self.parse_keyword_file()
        self.classify_data_by_keywords()
        self.create_classified_files()
        self.create_unclassified_files()
            
    def parse_protein_table(self):
        '''Parses Uniprot reference table'''
        with open(self._selected_protein_table, 'r') as in_fh:
            for entry in in_fh:
                if entry.startswith('Entry') or entry.startswith('UniprotID'):
                    self._file_header = entry
                else:
                    self._protein_data_set.add(entry)
        return self._protein_data_set, self._file_header
    
    def parse_keyword_file(self):
        '''Parse user derived keyoerds for clssififction'''
        with open(self._keyword_file, 'r') as in_fh:
            for entry in in_fh:
                self._keyword_set.add(entry.strip())
        return self._keyword_set
    
    def classify_data_by_keywords(self):
        '''create classified data'''
        for entry in self._protein_data_set:
            if len(entry.split('\t')) > 23:
                protein_name = entry.split('\t')[2]
                protein_go = entry.split('\t')[8]
                short_name = entry.split('\t')[18]
                full_name = entry.split('\t')[19]
                keyword_col = entry.split('\t')[20]
                domain_go = entry.split('\t')[21]
                for keyword in self._keyword_set:
                    self._keyword_selection(keyword, entry, protein_name)
                    self._keyword_selection(keyword, entry, protein_go)
                    self._keyword_selection(keyword, entry, short_name)
                    self._keyword_selection(keyword, entry, full_name)
                    self._keyword_selection(keyword, entry, keyword_col)
                    self._keyword_selection(keyword, entry, domain_go)
            else:
                # ShortName       FullName        DomainKeyword   DomainGo
                short_name = entry.split('\t')[3]
                full_name = entry.split('\t')[4]
                keyword_col = entry.split('\t')[5]
                domain_go = entry.split('\t')[6]
                for keyword in self._keyword_set:
                    self._keyword_selection(keyword, entry, short_name)
                    self._keyword_selection(keyword, entry, full_name)
                    self._keyword_selection(keyword, entry, keyword_col)
                    self._keyword_selection(keyword, entry, domain_go)
                
        return self._keyword_candidate_data
    
    def create_classified_files(self):
        '''Creates individual doamin file foreach keyword selection'''
        for keyword in self._keyword_candidate_data.keys():
            with open(self._classified_result_path+'/%s_selected_data.csv' %
                      keyword, 'w') as out_fh:
                out_fh.write(self._file_header)
                if len(self._keyword_candidate_data[keyword]) > 1:
                    for entry in self._keyword_candidate_data[keyword]:
                        out_fh.write(entry)
                        self._classified_data.add(entry)
                else:
                    out_fh.write(list(
                        self._keyword_candidate_data[keyword])[0])
                    self._classified_data.add(
                        list(self._keyword_candidate_data[keyword])[0])
        return self._classified_data
    
    def create_unclassified_files(self):
        '''Combines all the files in common'''
        for entry in self._protein_data_set:
            if entry not in self._classified_data:
                self._unclassified_data.add(entry.strip())
        if len(list(self._unclassified_data)) > 1:
            with open(
                self._classified_result_path+'/unclassified_selected_data.csv',
                    'w') as out_fh:
                out_fh.write(self._file_header)
                out_fh.write('\n'.join(list(self._unclassified_data)))
    
    def _keyword_selection(self, keyword, entry, info):
        domain_full_name = list(entry.lower().split(' '))
        check_keyword = keyword.lower()
        if '-' in check_keyword:
            ref_kw_list = check_keyword.split('-')
            new_kw_list = []
            for domain_name in domain_full_name:
                for string in ref_kw_list:
                    if string == 'rna':
                        if re.findall(r"(?:\s|^)%s(?=\s|$)" %
                                      '*rna', domain_name):
                            if string not in set(new_kw_list):
                                new_kw_list.append(string)
                    else:
                        if re.findall(r"(?:\s|^)%s(?=\s|$)" %
                                      string, domain_name):
                            if string not in set(new_kw_list):
                                new_kw_list.append(string)
            if str(ref_kw_list) == str(new_kw_list):
                self._keyword_candidate_data.setdefault(
                    keyword, set()).add(entry)
        else:
            for domain_name in domain_full_name:
                if re.findall(r"(?:\s|^)%s(?=\s|$)" %
                              check_keyword, domain_name):
                    self._keyword_candidate_data.setdefault(
                        keyword, set()).add(entry)



#! /usr/bin/env python
# Description = This script is designed to convert the file types.
# tab separated files to xlsx files

import os
import sys
import csv
import shutil
csv.field_size_limit(sys.maxsize)
try:
    from openpyxl.workbook import Workbook
except ImportError:
    print('Python package openpyxl is missing. Please install/update.\n'
          'Please ignore if you chose the output format as HTML')
    

def csv_to_xlsx(inpath, outpath):
    '''Converts table to excel'''
    allowed_format_list = ['csv', 'tab', 'tsv']
    for csv_file in os.listdir(inpath):
        if os.path.isfile(inpath+'/'+csv_file):
            if csv_file.split('.')[1] in allowed_format_list:
                sheet_name = csv_file.split('.')[0]
                workbook = Workbook()
                xlsxsheet = workbook.create_sheet(0)
                for i, row in enumerate(
                        csv.reader(open(inpath+'/'+csv_file), delimiter="\t"), 1):
                    for j, col in enumerate(row, 1):
                        xlsxsheet.cell(row=i, column=j).value = col
                workbook.save(open(outpath+'/'+sheet_name+'.xlsx', 'wb'))
            else:
                shutil.copyfile(inpath+'/'+csv_file, outpath+'/'+csv_file)
        elif os.path.isdir(inpath+'/'+csv_file):
            if not os.path.exists(outpath+'/'+csv_file):
                os.mkdir(outpath+'/'+csv_file)
            csv_to_xlsx(inpath+'/'+csv_file, outpath+'/'+csv_file)

            
def csv_to_html(inpath, outpath):
    '''Convert table to HTML, line break 13-16'''
    allowed_format_list = ['csv', 'tab', 'tsv']
    for csv_file in os.listdir(inpath):
        if os.path.isfile(inpath+'/'+csv_file):
            if csv_file.split('.')[1] in allowed_format_list:
                with open(outpath+'/'+csv_file.split('.')[0]+'.html',"w") as outfile:
                    outfile.write('\n'.join(['<!DOCTYPE html>', '<html lang="en">', '<head>',
                    '<meta charset="utf-8">',
                    '<link rel="stylesheet" type="text/css" href="http://ajax.aspnetcdn.com/ajax/jquery.dataTables/1.9.4/css/jquery.dataTables.css">',
                    '<script type="text/javascript" charset="utf8" src="http://ajax.aspnetcdn.com/ajax/jQuery/jquery-1.8.2.min.js"></script>',
                    '<script type="text/javascript" charset="utf8" src="http://ajax.aspnetcdn.com/ajax/jquery.dataTables/1.9.4/jquery.dataTables.min.js"></script>',
                    '<script type="text/javascript" language="javascript" class="init">',
                    '$(document).ready(function () {',
                    '\t$("#subscriptionlist").dataTable();',
                    '});', '</script>', '<style type="text/CSS">',
                    'table, th, td {', '\tborder: 1px solid #E8E8E8;',
                    '\tborder-collapse: collapse;', '\tmin-width:150px;',
                    '\ttext-align:center', '}', '</style>',
                    '</head>', '<div id="subscriptionsList">',
                    '<h2>Source file: %s</h2>' % csv_file,
                    '<h3>Source path: %s</h3>' % inpath, '<table id="subscriptionlist">\n']))
                    row_num = 0
                    table_string = ""
                    csv_fh = csv.reader(open(inpath+'/'+csv_file), delimiter="\t")
                    for i, row in enumerate(csv_fh):
                        if 'annotation_scoring' in csv_file:
                            if 'annotation_scoring_of_selected_data_filter' in csv_file:
                                for idx in range(1, 3):
                                    if len(list(row[idx])) > 50:
                                        row[idx] = '<br>'.join(
                                            split_str(row[idx], 50))
                            else:
                                for idx in range(13, 17):
                                    if len(list(row[idx])) > 50:
                                        row[idx] = '<br>'.join(
                                            split_str(row[idx], 50))
                        if i == 0:
                            outfile.write("<thead>\n<tr>\n<th>")
                            outfile.write('</th><th>'.join(row)+'\n')
                            outfile.write("</th>\n</tr>\n</thead>\n<tbody>\n")
                        else:
                            outfile.write("<tr>\n<td>")
                            outfile.write('</td><td>'.join(row)+'\n')
                            outfile.write("</td>\n</tr>\n")
                    outfile.write(
                        '\n'.join(['</tbody>', '</table>', '</div>',
                                   '</html>']))
            else:
                shutil.copyfile(inpath+'/'+csv_file, outpath+'/'+csv_file)
        elif os.path.isdir(inpath+'/'+csv_file):
            if not os.path.exists(outpath+'/'+csv_file):
                os.mkdir(outpath+'/'+csv_file)
            csv_to_html(inpath+'/'+csv_file, outpath+'/'+csv_file)


def split_str(seq, chunk, skip_tail=False):
    lst = []
    if chunk <= len(seq):
        lst.extend([seq[:chunk]])
        lst.extend(split_str(seq[chunk:], chunk, skip_tail))
    elif not skip_tail and seq:
        lst.extend([seq])
    return lst



#!/usr/bin/env python
# Description = Calculate feature based combined annotation scores

import os
import sys
from collections import defaultdict
from difflib import *

try:
    from Bio.Emboss.Applications import NeedleCommandline
except ImportError:
    print('Python package Biopython is missing. Please install/update.\n')
    sys.exit(0)
from itertools import combinations

try:
    import numpy as np
except ImportError:
    print('Python package numpy is missing. Please install/update.\n')
    sys.exit(0)
import random
try:
    from scipy.spatial import distance
    from scipy.cluster.hierarchy import *
    from scipy import stats
except ImportError:
    print('Python package scipy is missing. Please install/update.\n')
    sys.exit(0)

class ComputeCompositionDistance(object):
    def __init__(self, filtered_data,
                 cdd_pred_files,
                 outpath, needle_path):
        self._filtered_data = filtered_data
        self._cdd_pred_files = cdd_pred_files
        self._outpath = outpath
        self._needle_path = needle_path
        
        self._cdd_main_entry_dict = defaultdict(lambda: defaultdict(lambda: str()))
        self._final_entry_dict = {}
        self._total_score_dict = {}
        self._ref_value_dict = defaultdict(lambda : defaultdict(lambda : (defaultdict(lambda: float))))
        self._ref_t_stat = defaultdict(lambda : defaultdict(lambda : (defaultdict(lambda: float))))
        self._ref_pval = defaultdict(lambda : defaultdict(lambda : (defaultdict(lambda: float))))
        self._t_stat_distribution = defaultdict(lambda : defaultdict(lambda : (defaultdict(lambda: []))))
        self._pval_distribution = defaultdict(lambda : defaultdict(lambda : (defaultdict(lambda: []))))
        self._composition_dict = defaultdict(
            lambda : (defaultdict(lambda : defaultdict(float))))
        self._calc_type_list = ('Secondary_structure', 'Amino_acid_residue',
                                'Amino_acid_property', 'Dipeptide')
        self._all_distance_data = defaultdict(
            lambda : (defaultdict(lambda : defaultdict(float))))
        
        self._amino_acid_str = {
            'H': ['E', 'A', 'L', 'M', 'Q', 'K', 'R', 'H', 'Z'],
            'E': ['V', 'I', 'Y', 'C', 'W', 'F', 'T'],
            'C': ['G', 'N', 'P', 'S', 'D', 'B']
        }
        #1: alpha-helix (H), 2: beta-sheet(E), 3: turns and loops(C)
        #https://en.wikibooks.org/wiki/Structural_Biochemistry/Proteins/Structures
        
        self._amino_acid_list = ('A', 'C', 'D', 'E', 'F',
                                  'G', 'H', 'I', 'K', 'L',
                                  'M', 'N', 'P', 'Q', 'R',
                                  'S', 'T', 'V', 'W', 'Y',
                                  'O', 'U', 'X')
        
        #http://www.seas.upenn.edu/~cis535/Fall2004/HW/GCB535HW6b.pdf
        #C.Chothia, J. Mol. Biol., 105(1975)1-14 
        #A.A. Zamyatin, Prog. Biophys. Mol. Biol., 24(1972)107-123 
        #C. Tanford, Adv. Prot. Chem., 17(1962)69-165 
        #The Merck Index, Merck & Co. Inc., Nahway, N.J., 11(1989); CRC Handbook of Chem.& Phys., 
        #Cleveland, Ohio, 58(1977)
        #name: information from NIST Chemistry WebBook, three letter code: GIF, one letter code: VRML 
        
        #mass in dalton
        self._amino_acid_mass = {'A' : 71.08, 'C' : 103.15, 'D' : 115.08,
                                 'E' : 129.12, 'F' : 147.18, 'G' : 57.05,
                                 'H' : 137.14, 'I' : 113.16, 'K' : 128.17,
                                 'L' : 113.16, 'M' : 131.19, 'N' : 114.103,
                                 'P' : 97.12, 'Q' : 128.14, 'R' : 156.19,
                                 'S' : 87.08, 'T' : 101.11, 'V' : 99.14,
                                 'W' : 186.12, 'Y' : 163.18, 'U' : 150.039,
                                 'O' : 255.31}
        
        #pI at 25degree: Isoelectric point (pI) is a pH in which net charge of protein is 0
        self._amino_acid_isoelpt = {'A' : 6.107, 'C' : 5.02, 'D' : 2.98,
                                 'E' : 3.08, 'F' : 5.91, 'G' : 6.064,
                                 'H' : 7.64, 'I' : 6.038, 'K' : 9.47,
                                 'L' : 6.036, 'M' : 5.74, 'N' : '-',
                                 'P' : 6.3, 'Q' : '-', 'R' : 10.76,
                                 'S' : 5.68, 'T' : '-', 'V' : 6.002,
                                 'W' : 5.88, 'Y' : 163.18}
        #Solubility (g/100g)
        
        self._amino_acid_solubility = {'A' : 16.65, 'C' : 30, 'D' : 0.778,
                                 'E' : 0.864, 'F' : 2.965, 'G' : 24.99,
                                 'H' : 4.19, 'I' : 4.117, 'K' : 30,
                                 'L' : 2.426, 'M' : 3.381, 'N' : 3.53,
                                 'P' : 162.3, 'Q' : 2.5, 'R' : 15,
                                 'S' : 5.023, 'T' : 30, 'V' : 8.85,
                                 'W' : 1.136, 'Y' : 0.0453}
        
        self._van_der_waal_volume = {'A' : 67, 'C' : 86, 'D' : 91,
                                 'E' : 109, 'F' : 135, 'G' : 48,
                                 'H' : 118, 'I' : 124, 'K' : 135,
                                 'L' : 124, 'M' : 124, 'N' : 96,
                                 'P' : 90, 'Q' : 114, 'R' : 148,
                                 'S' : 73, 'T' : 93, 'V' : 105,
                                 'W' : 163, 'Y' : 141}
        
        self._acid_dissociation_pka = {'A' : 0, 'C' : 8.18, 'D' : 3.9,
                                 'E' : 4.07, 'F' : 0, 'G' : 0,
                                 'H' : 6.04, 'I' : 0, 'K' : 10.54,
                                 'L' : 0, 'M' : 0, 'N' : 0,
                                 'P' : 0, 'Q' : 0, 'R' : 12.48,
                                 'S' : 5.68, 'T' : 5.53, 'U':  5.73,
                                 'V' : 0, 'W' : 163, 'Y' : 141}
        
        self._ph_value = {'C' : 'A', 'D' : 'A', 'E' : 'A', 'H' : 'wB', 'K' : 'B',
                    'O' : 'wB', 'Q' : 'wB', 'R' : 'sB', 'S' : 'wA', 'T' : 'wA',
                     'U':  'A', 'W' : 'wB', 'Y' : 'wA'}
        
        self._solvent_accessibility = {'M' : 'm', 'P' : 'm', 'S' : 'm',
                                       'T' : 'm', 'H' : 'm', 'Y' : 'm',
                                       'R' : 'h', 'K' : 'h', 'Q' : 'h',
                                       'E' : 'h', 'N' : 'h', 'D' : 'h'}
        
        self._charge = {'D' : 'N', 'E' : 'N', 'R' : 'P', 'K' : 'P', 'H' : 'P',
                        'N' : 'U', 'Q' : 'U', 'C' : 'U', 'T' : 'U', 'S' : 'U',
                        'Y' : 'U',}
        
        self._amino_acid_ref = {
            'a_polar' : ['Y', 'N', 'Q', 'T', 'S', 'D', 'E', 'R',
                         'K', 'H', 'Y', 'O', 'W', 'C'],
            'b_non_polar' : ['F', 'G', 'A', 'V', 'P', 'L', 'I', 'M'],
            'c_neutral' : ['T', 'E', 'G', 'S', 'Q', 'D', 'H'],
            'd_hydrophobic' : ['C', 'Y', 'A', 'L', 'I', 'F', 'W', 'V', 'M'],
            'e_hydrophilic' : ['R', 'K', 'N', 'H', 'P', 'E', 'D'],
            'f_van_der_waal=0-4' : ['G', 'A', 'S', 'P', 'D', 'N',
                                    'V', 'E', 'Q', 'I', 'L'],
            'g_van_der_waal>4' : ['M', 'H', 'K', 'F', 'R', 'Y', 'W'],
            'h_mid_polarity' : ['P', 'A', 'T', 'G', 'S'],
            'i_high_polarity' : ['H', 'Q', 'R', 'K', 'N', 'E', 'D'],
            'j_negative' : ['D', 'E'],
            'k_positive' : ['R', 'K', 'H'],
            'l_uncharged' : ['N', 'Q', 'C', 'T', 'S', 'Y'],
            'm_aliphatic' : ['A', 'I', 'L', 'V', 'G'],
            'n_aromatic' : ['F', 'H', 'W', 'Y'],
            'o_basic' : ['K', 'R', 'H', 'W', 'Q', 'O'],
            'p_acidic' : ['C', 'D', 'E', 'U', 'Y', 'T', 'S'],
            'q_tiny' : ['A', 'C', 'G', 'S', 'U'],
            'r_small' : ['A', 'C', 'D', 'G', 'N', 'P', 'S', 'T', 'U'],
            's_large' : ['V', 'L', 'I', 'M', 'P', 'F', 'W'],
            't_mid_solvent_accebility' : ['M', 'P', 'S', 'T', 'H', 'Y'],
            'u_high_solvent_access' : ['R', 'K', 'Q', 'E', 'N', 'D'],
            'v_essential' : ['F', 'H', 'I', 'K', 'L', 'M', 'T', 'V', 'W'],
            'w_non_essential' : ['A', 'D', 'N', 'O', 'P', 'Q', 'S', 'U']
        }
        
    def streamline_annotation_scoring(self):
        self.parse_filtered_data()
        self.summarize_rps_analysis_result_files()
        
    def parse_filtered_data(self):
        '''Parse the selected data'''
        with open(self._filtered_data, 'r') as in_fh:
            for entry in in_fh:
                if entry.startswith('Entry'):
                    self._header_filter_data = entry.strip()
                else:
                    filter_data_annotation = FilteredData(
                        entry.strip().split('\t'))
                    if filter_data_annotation.resource == 'CDD':
                        cdd_main = "%s\t%s\t%s\t%s" % (
                        filter_data_annotation.resource_id,
                        filter_data_annotation.domain_id,
                        filter_data_annotation.start,
                        filter_data_annotation.stop)
                        self._cdd_main_entry_dict[
                            filter_data_annotation.uid][cdd_main] = entry.strip()
        return self._cdd_main_entry_dict, self._header_filter_data
    
    def summarize_rps_analysis_result_files(self):
        '''summarize all the RPS-BLAST results in dictionary'''
        summary_file = open(self._outpath+'/annotation_scoring_of_selected_data.csv', 'w')
        filter3_file = open(self._outpath+'/annotation_scoring_of_selected_data_filter3.csv', 'w')
        filter4_file = open(self._outpath+'/annotation_scoring_of_selected_data_filter4.csv', 'w')
        filter3_dist_file = open(self._outpath+'/filter3_pval_distribution.csv', 'w')
        filter4_dist_file = open(self._outpath+'/filter4_pval_distribution.csv', 'w')
        self._result_detail_dict = defaultdict(lambda: defaultdict(
            lambda: defaultdict(lambda: str)))
        summary_header = ('\t'.join([
        '\t'.join(self._header_filter_data.split('\t')[0:5]),
        '\t'.join(self._header_filter_data.split('\t')[15:20]),
        '\t'.join(self._header_filter_data.split('\t')[23:26]),
        'ReferenceSequence', 'QuerySequence',
        'ReferenceSecStr (H:alpha-helix, E:beta-sheet, C:turns&loops)',
        'QuerySecStr (H:alpha-helix, E:beta-sheet, C:turns&loops)',
        'Coverage','Identity', 'Similarity','Gap',
        'CoverageByDomainLength (filter-1)', 'IdentityByCoverage (filter-1)',
        'SimilarityByCoverage (filter-1)', 'UngappedByCoverage (filter-1)',
        'MolecularMass (filter-2)', 'IsoelectricPoint (filter-2)',
        'Solubility (filter-2)', 'VanDerWaalValue (filter-2)',
        'PkaValue (filter-2)', 'AA-RODistance(filter-3)',
        'SecStr-RODistance (filter-3)', 'PhysiChemiComp-EDistance (filter-4)',
        'DipeptideComp-EDistance (filter-4)', 'TripeptideComp-EDistance (filter-4)',
        'Average filter-1 (Predicted properties (PP))',
        'Average filter-2 (Chemical properties)',
        'Average filter-3 (Needleman Wunsch sequence similarity)',
        'Average filter-4 (Euclidean distance for composition)',
        'Bayesian Score (based on all the filters)'])+'\n')
        filter3_file.write('\t'.join(['ProteinId',
        'ReferenceSequence', 'QuerySequence', 'AA-RODistance (filter-3)',
        'AA-RODistance - t_statistics', 'AA-RODistance -P_value',
        'SecStr-RODistance (filter-3)', 'SecStr-RODistance - t_statistics',
        'SecStr-RODistance - P_value', 'Average filter-3 (Ratcliff Obershelp distance of PP)',
        'Average filter-3 - t_statistics', 'Average filter-3 - P_value'])+'\n')
        filter4_file.write('\t'.join(['ProteinId', 'ReferenceSequence', 'QuerySequence',
        'PhysiChemiComp-EDistance (filter-4)', 'PhysiChemiComp-EDistance - t_statistics',
        'PhysiChemiComp-EDistance - P_value', 'DipeptideComp-EDistance (filter-4)',
        'DipeptideComp-EDistance - t_statistics', 'DipeptideComp-EDistance - P_value',
        'TripeptideComp-EDistance (filter-4)', 'TripeptideComp-EDistance - t_statistics',
        'TripeptideComp-EDistance - P_value', 'Average filter-4 (Euclidean distance of PP)',
        'Average filter-4 - t_statistics', 'Average filter-4 -P_value'])+'\n')
        filter3_dist_file.write('\t'.join(['ProteinId',
        'Ref-AA-RODistance - t_statistics', 'Ref-AA-RODistance -P_value',
        'Random-AA-RODistance - t_statistics', 'Random-AA-RODistance -P_value',
        'Ref-SecStr-RODistance - t_statistics', 'Ref-SecStr-RODistance - P_value',
        'Random-SecStr-RODistance - t_statistics', 'Random-SecStr-RODistance - P_value',
        'Ref-Average filter-3 - t_statistics', 'Ref-Average filter-3 - P_value',
        'Random-Average filter-3 - t_statistics', 'Random-Average filter-3 - P_value'])+'\n')
        filter4_dist_file.write('\t'.join(['ProteinId', 
        'Ref-PhysiChemiComp-EDistance - t_statistics', 'Ref-PhysiChemiComp-EDistance - P_value',
        'Random-PhysiChemiComp-EDistance - t_statistics', 'Random-PhysiChemiComp-EDistance - P_value',
        'Ref-DipeptideComp-EDistance - t_statistics', 'Ref-DipeptideComp-EDistance - P_value',
        'Random-DipeptideComp-EDistance - t_statistics', 'Random-DipeptideComp-EDistance - P_value',
        'Ref-TripeptideComp-EDistance - t_statistics', 'Ref-TripeptideComp-EDistance - P_value',
        'Random-TripeptideComp-EDistance - t_statistics', 'Random-TripeptideComp-EDistance - P_value',
        'Ref-Average filter-4 - t_statistics', 'Ref-Average filter-4 -P_value',
        'Random-Average filter-4 - t_statistics', 'Random-Average filter-4 -P_value'])+'\n')
        summary_file.write(summary_header)
        for rps_result_file in os.listdir(self._cdd_pred_files):
            protein_id = rps_result_file.split('.')[0]
            if protein_id in self._cdd_main_entry_dict.keys():
                individual_rps_result_fh = open(
                    self._cdd_pred_files+'/'+rps_result_file)
                for individual_rps_result_section in individual_rps_result_fh.read(
                    ).split('>gnl'):
                    if individual_rps_result_section.startswith('|CDD|'):
                        location = self._get_location(individual_rps_result_section)
                        stat_data = individual_rps_result_section.split("Score = ")
                        for individual_rps_result in stat_data[0].split('\n'):
                            if individual_rps_result.startswith('|CDD|'):
                                for entry in individual_rps_result.split('\n'):
                                    if '|CDD|' in entry:
                                        pssm_id = entry.split('|CDD|')[1].split(' ')[0]
                                        parent_id = entry.split(',')[0].split(' ')[1]
                                        if 'smart' in parent_id:
                                            domain_id = parent_id.replace(
                                                'smart', 'SM')
                                        elif 'pfam' in parent_id:
                                            domain_id = parent_id.replace(
                                                'pfam', 'PF')
                                        else:
                                            domain_id = parent_id
                                        cdd_main = ('%s\t%s\t%s' % (
                                            pssm_id, domain_id, location))
                                        if cdd_main in self._cdd_main_entry_dict[protein_id].keys():
                                            cdd_main_entry = self._cdd_main_entry_dict[protein_id][cdd_main]
                                            cdd_main_entry1 = '\t'.join(
                                                cdd_main_entry.split('\t')[0:5])
                                            cdd_main_entry2 = '\t'.join(
                                                cdd_main_entry.split('\t')[15:20])
                                            cdd_main_entry3 = '\t'.join(
                                                cdd_main_entry.split('\t')[23:26])
                                            
                                            cdd_main_entry_items = FilteredData(cdd_main_entry.split('\t'))
                                            coverage = float(cdd_main_entry_items.cover_length)
                                            coverage_percent = (float(cdd_main_entry_items.cover_length)/
                                                                float(cdd_main_entry_items.length))
                                            if coverage_percent > 1:
                                                coverage_percent = 1
                                            identity = cdd_main_entry_items.identity
                                            identity_percent = (float(cdd_main_entry_items.identity.split(
                                                '/')[0])/float(cdd_main_entry_items.cover_length))
                                            similarity = cdd_main_entry_items.similarity
                                            similarity_percent = (float(cdd_main_entry_items.similarity.split(
                                                '/')[0])/float(cdd_main_entry_items.cover_length))
                                            if not cdd_main_entry_items.gaps == 'None':
                                                gaps = cdd_main_entry_items.gaps
                                                ungap_percent = 1-(float(cdd_main_entry_items.gaps.split(
                                                '/')[0])/float(cdd_main_entry_items.cover_length))
                                            else:
                                                gaps = 0
                                                ungap_percent = 1
                                            for each_stat_group in stat_data[1:]:
                                                common_seq = []
                                                query_seq, subject_seq =  self._compile_cdd_stat(
                                                    each_stat_group)
                                                seq_distance = 0
                                                needle_cline_seq = NeedleCommandline(self._needle_path, asequence="asis:"+query_seq,
                                                bsequence="asis:"+subject_seq, gapopen=0, gapextend=0, outfile="stdout")
                                                stdout, stderr = needle_cline_seq()
                                                for entry_seq in stdout.split('\n'):
                                                    if entry_seq.startswith("# Similarity: "):
                                                        seq_distance = float(entry_seq.strip().split(
                                                        '(')[1].split('%)')[0])/100
                                                query_dp_comp = self._compute_dp_comp(query_seq)
                                                subject_dp_comp = self._compute_dp_comp(subject_seq)
                                                dp_comp_distance = 1-distance.euclidean(
                                                    query_dp_comp, subject_dp_comp)/100
                                                
                                                query_tp_comp = self._compute_tp_comp(query_seq)
                                                subject_tp_comp = self._compute_tp_comp(subject_seq)
                                                tp_comp_distance = 1-distance.euclidean(
                                                    query_tp_comp, subject_tp_comp)/100
                                                
                                                query_mass, query_pi, query_solub, \
                                                query_vdw, query_pka = self._compute_properties(
                                                    query_seq)
                                                subject_mass, subject_pi, subject_solub, \
                                                subject_vdw, subject_pka = self._compute_properties(
                                                    subject_seq)
                                                 
                                                mass_score, mass_entry = self._calculate_score(
                                                    subject_mass, query_mass)
                                                pi_score, pi_entry = self._calculate_score(
                                                    subject_pi, query_pi)
                                                solub_score, solub_entry = self._calculate_score(
                                                    subject_solub, query_solub)
                                                vdw_score, vdw_entry = self._calculate_score(
                                                    subject_vdw, query_vdw)
                                                pka_score, pka_entry = self._calculate_score(
                                                    subject_pka, query_pka)
                                                
                                                query_ss = self._compute_sec_str(query_seq)
                                                subject_ss = self._compute_sec_str(subject_seq)
                                                #ss_distance = difflib.SequenceMatcher(
                                                #    None, query_ss, subject_ss).ratio()
                                                ss_distance = 0
                                                needle_cline_ss = NeedleCommandline(asequence="asis:"+query_ss,
                                                bsequence="asis:"+subject_ss, gapopen=0, gapextend=0, outfile="stdout")
                                                stdout, stderr = needle_cline_ss()
                                                for entry_ss in stdout.split('\n'):
                                                    if entry_ss.startswith("# Similarity: "):
                                                        ss_distance = float(entry_ss.strip().split(
                                                        '(')[1].split('%)')[0])/100
                                                query_pcn, query_pcn_comp = self._compute_group_aac(
                                                    query_seq)
                                                subject_pcn, subject_pcn_comp = self._compute_group_aac(
                                                    subject_seq)
                                                pcn_comp_distance = 1-distance.euclidean(
                                                    query_pcn_comp, subject_pcn_comp)/100
                                                filter1 = (coverage_percent+identity_percent+
                                                        similarity_percent+ungap_percent)/4
                                                filter2 = (float(mass_score)+ float(
                                                    pi_score)+ float(solub_score)+ float(vdw_score)+
                                                    float(pka_score))/5
                                                filter3 = (float(seq_distance)+
                                                         float(ss_distance))/2
                                                filter4 = (float(dp_comp_distance)+
                                                           float(tp_comp_distance)+
                                                         float(pcn_comp_distance))/3
                                                bayesian_score = (filter1 + filter2 + filter3 + filter4)/4
                                                self._ref_value_dict[protein_id][domain_id]['seq_distance'] = float(seq_distance)
                                                self._ref_value_dict[protein_id][domain_id]['ss_distance'] = float(ss_distance)
                                                #self._ref_value_dict[protein_id][domain_id]['pcn_distance'] = float(pcn_distance)
                                                self._ref_value_dict[protein_id][domain_id]['dp_comp_distance'] = float(dp_comp_distance)
                                                self._ref_value_dict[protein_id][domain_id]['tp_comp_distance'] = float(tp_comp_distance)
                                                self._ref_value_dict[protein_id][domain_id]['pcn_comp_distance'] = float(pcn_comp_distance)
                                                self._ref_value_dict[protein_id][domain_id]['filter-3'] = "%.4f" % float(filter3)
                                                self._ref_value_dict[protein_id][domain_id]['filter-4'] = "%.4f" % float(filter4)
                                                self._get_t_stats(protein_id, domain_id, query_seq, subject_seq)
                                                final_entry = "\t".join(map(str, [cdd_main_entry1,
                                                    cdd_main_entry2, cdd_main_entry3,
                                                    subject_seq, query_seq, subject_ss,
                                                    query_ss, coverage, identity,
                                                    similarity, gaps, 
                                                    "%.4f" % float(coverage_percent),
                                                    "%.4f" % float(identity_percent),
                                                    "%.4f" % float(similarity_percent),
                                                    "%.4f" % float(ungap_percent),
                                                    mass_entry, pi_entry,
                                                    solub_entry, vdw_entry, pka_entry,
                                                    "%.4f" % float(seq_distance),
                                                    "%.4f" % float(ss_distance),
                                                    "%.4f" % float(pcn_comp_distance),
                                                    "%.4f" % float(dp_comp_distance),
                                                    "%.4f" % float(tp_comp_distance),
                                                    "%.4f" % float(filter1), "%.4f" % float(
                                                        filter2), "%.4f" % float(filter3),
                                                    "%.4f" % float(filter4), bayesian_score]))
                                                summary_file.write(final_entry+'\n')
                                                filter3_file.write('\t'.join(map(str, [protein_id,
                                                    subject_seq, query_seq,
                                                    "%.4f" % float(seq_distance),
                                                    self._ref_t_stat[protein_id][domain_id]['seq_distance'],
                                                    self._ref_pval[protein_id][domain_id]['seq_distance'],
                                                    "%.4f" % float(ss_distance),
                                                    self._ref_t_stat[protein_id][domain_id]['ss_distance'],
                                                    self._ref_pval[protein_id][domain_id]['ss_distance'],
                                                    "%.4f" % float(filter3), self._ref_t_stat[protein_id][domain_id]['filter-3'],
                                                    self._ref_pval[protein_id][domain_id]['filter-3']]))+'\n')
                                                filter4_file.write('\t'.join(map(str, [protein_id,
                                                    subject_seq, query_seq,
                                                    "%.4f" % float(pcn_comp_distance),
                                                    self._ref_t_stat[protein_id][domain_id]['pcn_comp_distance'],
                                                    self._ref_pval[protein_id][domain_id]['pcn_comp_distance'],
                                                    "%.4f" % float(dp_comp_distance),
                                                    self._ref_t_stat[protein_id][domain_id]['dp_comp_distance'],
                                                    self._ref_pval[protein_id][domain_id]['dp_comp_distance'],
                                                    "%.4f" % float(tp_comp_distance),
                                                    self._ref_t_stat[protein_id][domain_id]['tp_comp_distance'],
                                                    self._ref_pval[protein_id][domain_id]['tp_comp_distance'],
                                                    "%.4f" % float(filter4), self._ref_t_stat[protein_id][domain_id]['filter-4'],
                                                    self._ref_pval[protein_id][domain_id]['filter-4']]))+'\n')
                                                filter3_dist_file.write('\t'.join(map(str, [protein_id,
                                                self._ref_t_stat[protein_id][domain_id]['seq_distance'],
                                                self._ref_pval[protein_id][domain_id]['seq_distance'],
                                                ','.join(self._t_stat_distribution[protein_id][domain_id]['seq_distance']),
                                                ','.join(self._pval_distribution[protein_id][domain_id]['seq_distance']),
                                                self._ref_t_stat[protein_id][domain_id]['ss_distance'],
                                                self._ref_pval[protein_id][domain_id]['ss_distance'],
                                                ','.join(self._t_stat_distribution[protein_id][domain_id]['ss_distance']),
                                                ','.join(self._pval_distribution[protein_id][domain_id]['ss_distance']),
                                                self._ref_t_stat[protein_id][domain_id]['filter-3'],
                                                self._ref_pval[protein_id][domain_id]['filter-3'],
                                                ','.join(self._t_stat_distribution[protein_id][domain_id]['filter-3']),
                                                ','.join(self._pval_distribution[protein_id][domain_id]['filter-3'])]))+'\n')
                                                filter4_dist_file.write('\t'.join(map(str, [protein_id,
                                                self._ref_t_stat[protein_id][domain_id]['pcn_comp_distance'],
                                                self._ref_pval[protein_id][domain_id]['pcn_comp_distance'],
                                                ','.join(self._t_stat_distribution[protein_id][domain_id]['pcn_comp_distance']),
                                                ','.join(self._pval_distribution[protein_id][domain_id]['pcn_comp_distance']),
                                                self._ref_t_stat[protein_id][domain_id]['dp_comp_distance'],
                                                self._ref_pval[protein_id][domain_id]['dp_comp_distance'],
                                                ','.join(self._t_stat_distribution[protein_id][domain_id]['dp_comp_distance']),
                                                ','.join(self._pval_distribution[protein_id][domain_id]['dp_comp_distance']),
                                                self._ref_t_stat[protein_id][domain_id]['tp_comp_distance'],
                                                self._ref_pval[protein_id][domain_id]['tp_comp_distance'],
                                                ','.join(self._t_stat_distribution[protein_id][domain_id]['tp_comp_distance']),
                                                ','.join(self._pval_distribution[protein_id][domain_id]['tp_comp_distance']),
                                                self._ref_t_stat[protein_id][domain_id]['filter-4'],
                                                self._ref_pval[protein_id][domain_id]['filter-4'],
                                                ','.join(self._t_stat_distribution[protein_id][domain_id]['filter-4']),
                                                ','.join(self._pval_distribution[protein_id][domain_id]['filter-4'])]))+'\n')
                individual_rps_result_fh.close()
        filter3_file.close()
        filter4_file.close()
        filter3_dist_file.close()
        filter3_dist_file.close()
        summary_file.close()
        
    def _get_t_stats(self, protein_id, domain_id, query_seq, ref_subject):
        ''''''
        value_dict = {}
        ref_subject_list = []
        ref_subject_list.append(ref_subject)
        query_aac = self._compute_each_aac(query_seq)
        query_ss = self._compute_sec_str(query_seq)
        query_dp_comp = self._compute_dp_comp(query_seq)
        query_tp_comp = self._compute_tp_comp(query_seq)
        query_pcn, query_pcn_comp = self._compute_group_aac(
            query_seq)
        value_dict.setdefault('seq_distance', []).append(
            self._ref_value_dict[protein_id][domain_id]['seq_distance'])
        value_dict.setdefault('ss_distance', []).append(
            self._ref_value_dict[protein_id][domain_id]['ss_distance'])
        #value_dict.setdefault('pcn_distance', []).append(
        #    self._ref_value_dict[protein_id][domain_id]['pcn_distance'])
        value_dict.setdefault('dp_comp_distance', []).append(
            self._ref_value_dict[protein_id][domain_id]['dp_comp_distance'])
        value_dict.setdefault('tp_comp_distance', []).append(
            self._ref_value_dict[protein_id][domain_id]['tp_comp_distance'])
        value_dict.setdefault('pcn_comp_distance', []).append(
            self._ref_value_dict[protein_id][domain_id]['pcn_comp_distance'])
        value_dict.setdefault('filter-3', []).append(
            self._ref_value_dict[protein_id][domain_id]['filter-3'])
        value_dict.setdefault('filter-4', []).append(
            self._ref_value_dict[protein_id][domain_id]['filter-4'])
        for i in range(1, 501):
            subject_list = list(ref_subject)
            random.shuffle(subject_list, random.random)
            subject_seq = ''.join(subject_list)
            ref_subject_list.append(subject_seq)
            seq_distance = 0
            needle_cline_seq = NeedleCommandline(asequence="asis:"+query_seq,
            bsequence="asis:"+subject_seq, gapopen=0, gapextend=0, outfile="stdout")
            stdout, stderr = needle_cline_seq()
            for entry_seq in stdout.split('\n'):
                if entry_seq.startswith("# Similarity: "):
                    seq_distance = float(entry_seq.strip().split(
                        '(')[1].split('%)')[0])/100
            subject_ss = self._compute_sec_str(subject_seq)
            ss_distance = 0
            needle_cline_ss = NeedleCommandline(asequence="asis:"+query_ss,
            bsequence="asis:"+subject_ss, gapopen=0, gapextend=0, outfile="stdout")
            stdout, stderr = needle_cline_ss()
            for entry_ss in stdout.split('\n'):
                if entry_ss.startswith("# Similarity: "):
                    ss_distance = float(entry_ss.strip().split(
                    '(')[1].split('%)')[0])/100
            subject_dp_comp = self._compute_dp_comp(subject_seq)
            dp_comp_distance = 1-distance.euclidean(
                query_dp_comp, subject_dp_comp)/100
            subject_tp_comp = self._compute_tp_comp(subject_seq)
            tp_comp_distance = 1-distance.euclidean(
                query_tp_comp, subject_tp_comp)/100
            subject_pcn, subject_pcn_comp = self._compute_group_aac(
                subject_seq)
            pcn_comp_distance = 1-distance.euclidean(
                query_pcn_comp, subject_pcn_comp)/100
            filter3 = (float(seq_distance)+
                    float(ss_distance))/2
            filter4 = (float(dp_comp_distance)+
                      float(tp_comp_distance)+
                    float(pcn_comp_distance))/3
            value_dict.setdefault('seq_distance', []).append(
                float(seq_distance))
            value_dict.setdefault('ss_distance', []).append(
                float(seq_distance))
            #value_dict.setdefault('pcn_distance', []).append(
            #    float(seq_distance))
            value_dict.setdefault('dp_comp_distance', []).append(
                float(seq_distance))
            value_dict.setdefault('tp_comp_distance', []).append(
                float(seq_distance))
            value_dict.setdefault('pcn_comp_distance', []).append(
                float(seq_distance))
            value_dict.setdefault('filter-3', []).append(filter3)
            value_dict.setdefault('filter-4', []).append(filter4)
        for value_key in sorted(value_dict.keys()):
            value_array = np.asarray(value_dict[value_key], float)
            t_stat, pval =  stats.ttest_1samp(value_array,
                        float(self._ref_value_dict[protein_id][domain_id][value_key]))
            self._ref_t_stat[protein_id][domain_id][value_key] = '%.4f' % float(t_stat)
            self._ref_pval[protein_id][domain_id][value_key] = '%.4f' % float(pval)
            for each_val in sorted(value_array):
                pval, t_stat =  stats.ttest_1samp(value_array,
                                                 each_val)
                self._t_stat_distribution[protein_id][domain_id][value_key].append(
                    '%.4f' % t_stat)
                self._pval_distribution[protein_id][domain_id][value_key].append(
                    '%.4f' % pval)
        return self._ref_t_stat, self._ref_pval, \
        self._t_stat_distribution, self._pval_distribution


    def _get_location(self, each_stat_group):
        '''Get locations of predicted domains'''
        start_list = []
        stop_list = []
        for each_stat_data in each_stat_group.split('\n'):
            if 'Expect' in each_stat_data:
                score = 'Score='+each_stat_data.split(',')[0]
                expect = 'Expect='+each_stat_data.split('Expect = '
                                                        )[1].strip()
            if 'Identities = ' in each_stat_data:
                identities = 'Identities='+each_stat_data.split(
                    'Identities = ')[1].split(',')[0]
                try:
                    gaps = 'Gaps='+each_stat_data.split(
                        'Gaps = ')[1].strip()
                    positives = 'Positives='+each_stat_data.split(
                        'Positives = ')[1].split(',')[0]
                except:
                    gaps = 'Gaps=None'
                    positives = 'Positives='+each_stat_data.split(
                        'Positives = ')[1].strip()
            if 'Query:' in each_stat_data:
                start_list.append(each_stat_data.split(' ')[1])
                stop_list.append(each_stat_data.split(' ')[-1].strip())
        start = start_list[0]
        stop = stop_list[-1]
        location = "%s\t%s" % (start, stop)
        return location

    def _compile_cdd_stat(self, each_stat_group):
        query_seq = []
        subject_seq = []
        for each_stat_data in each_stat_group.split('\n'):
            if 'Query:' in each_stat_data:
                query_seq.append(' '.join(
                    each_stat_data.strip().split()).split(' ')[2])
            if 'Sbjct:' in each_stat_data:
                subject_seq.append(' '.join(
                    each_stat_data.strip().split()).split(' ')[2])
        return ''.join(query_seq), ''.join(subject_seq)
    
    def _compute_properties(self, aa_list):
        ''''''
        prot_mass = 0
        prot_pi = 0
        prot_solub = 0
        prot_vdw = 0
        prot_pka = 0
        for aa in aa_list:
            try:
                prot_mass += self._amino_acid_mass[aa]
                prot_pi += self._amino_acid_isoelpt[aa]
                prot_solub += self._amino_acid_solubility[aa]
                prot_vdw += self._van_der_waal_volume[aa]
                prot_pka += self._acid_dissociation_pka[aa]
            except:
                pass
        return prot_mass, prot_pi, prot_solub, prot_vdw, prot_pka
    
    def _calculate_score(self, subject_val, query_val):
        '''calculatd scores of similarity between query and subject'''
        score = ''
        min_query_val = query_val-(query_val*15//100)
        max_query_val = query_val+(query_val*15//100)
        if subject_val >= min_query_val and subject_val <= max_query_val:
            score = 1
        else:
            score = 0
        score_entry = '%s (%.4f, %.4f)' % (score,
                    float(query_val), float(subject_val))
        return score, score_entry
        
    def _compute_sec_str(self, seq_line):
        '''Computes 3-state secondary structure'''
        str_list = []
        seq_dict = defaultdict(
            lambda: defaultdict(lambda: defaultdict()))
        for aa in seq_line:
            str_val = ''
            if aa == 'J':
                str_val = 'x'
            elif aa == 'X':
                str_val = 'x'
            else:
                for sec_str in self._amino_acid_str.keys():
                    if aa in self._amino_acid_str[sec_str]:
                        str_val = sec_str
            str_list.append(str_val)
        return ''.join(str_list)
        
    def _compute_each_aac(self, aa_list):
        ''''''
        aa_comp = []
        amino_acid_count = {
            'A' : 0, 'C' : 0, 'D' : 0, 'E' : 0, 'F' : 0,
            'G' : 0, 'H' : 0, 'I' : 0, 'K' : 0, 'L' : 0,
            'M' : 0, 'N' : 0, 'P' : 0, 'Q' : 0, 'R' : 0, '-' : 0,
            'S' : 0, 'T' : 0, 'V' : 0, 'W' : 0, 'Y' : 0, 'b' : 0,
            'O' : 0, 'U' : 0, 'X' : 0, 'B' : 0, 'Z' : 0, 'J' : 0
        }
        length = len(aa_list)
        
        for aa in aa_list:
            amino_acid_count[aa] = int(amino_acid_count[aa]) + 1
        if amino_acid_count['B'] > 0 or amino_acid_count['b'] > 0:
            amino_acid_count['D'] = int(amino_acid_count['D']
                                        ) + amino_acid_count['B']/2
            amino_acid_count['N'] = int(amino_acid_count['N']
                                        ) + amino_acid_count['B']/2
        if amino_acid_count['Z'] > 0:
            amino_acid_count['E'] = int(amino_acid_count['E']
                                        ) + amino_acid_count['Z']/2
            amino_acid_count['Q'] = int(amino_acid_count['Q']
                                        ) + amino_acid_count['Z']/2
        if amino_acid_count['J'] > 0:
            amino_acid_count['I'] = int(amino_acid_count['I']
                                        ) + amino_acid_count['J']/2
            amino_acid_count['L'] = int(amino_acid_count['L']
                                        ) + amino_acid_count['J']/2
        for aa_count in sorted(self._amino_acid_list):
            if not float(amino_acid_count[aa_count]) == 0:
                aa_comp.append(float(amino_acid_count[aa_count]
                                     )*100/int(length))
            else:
                aa_comp.append(0)
        return aa_comp
            
    def _compute_group_aac(self, aa_seq):
        ''''''
        amino_acid_count = {
        'a_polar' : 0, 'b_non_polar' : 0, 'c_neutral' : 0,
        'd_hydrophobic' : 0, 'e_hydrophilic' : 0,
        'f_van_der_waal=0-4' : 0, 'g_van_der_waal>4' : 0,
        'h_mid_polarity' : 0, 'i_high_polarity' : 0,
        'j_negative' : 0, 'k_positive' : 0, 'l_uncharged' : 0,
        'm_aliphatic' : 0, 'n_aromatic' : 0,
        'o_basic' : 0, 'p_acidic' : 0,
        'q_tiny' : 0, 'r_small' : 0, 's_large' : 0,
        't_mid_solvent_accebility' : 0, 'u_high_solvent_access' : 0,
        'v_essential' : 0, 'w_non_essential' : 0
        }
        aac_list = []
        aac_values = []
        length = len(aa_seq)
        for aa in aa_seq:
            if aa == 'B':
                for aa_type in amino_acid_count.keys():
                    if 'D' in self._amino_acid_ref[aa_type]:
                        aac_list.append(aa_type.split('_')[0])
                        amino_acid_count[aa_type] = int(
                            amino_acid_count[aa_type]) + 0.5
                for aa_type in amino_acid_count.keys():
                    if 'N' in self._amino_acid_ref[aa_type]:
                        aac_list.append(aa_type.split('_')[0])
                        amino_acid_count[aa_type] = int(
                            amino_acid_count[aa_type]) + 0.5
            elif aa == 'Z':
                for aa_type in amino_acid_count.keys():
                    if 'E' in self._amino_acid_ref[aa_type]:
                        aac_list.append(aa_type.split('_')[0])
                        amino_acid_count[aa_type] = int(
                            amino_acid_count[aa_type]) + 0.5
                for aa_type in amino_acid_count.keys():
                    if 'Q' in self._amino_acid_ref[aa_type]:
                        aac_list.append(aa_type.split('_')[0])
                        amino_acid_count[aa_type] = int(
                            amino_acid_count[aa_type]) + 0.5
            elif aa == 'J':
                for aa_type in amino_acid_count.keys():
                    if 'I' in self._amino_acid_ref[aa_type]:
                        aac_list.append(aa_type.split('_')[0])
                        amino_acid_count[aa_type] = int(
                            amino_acid_count[aa_type]) + 0.5
                for aa_type in amino_acid_count.keys():
                    if 'L' in self._amino_acid_ref[aa_type]:
                        aac_list.append(aa_type.split('_')[0])
                        amino_acid_count[aa_type] = int(
                            amino_acid_count[aa_type]) + 0.5
            elif aa == 'X':
                aac_list.append('X')
            else:
                for aa_type in amino_acid_count.keys():
                    if aa in self._amino_acid_ref[aa_type]:
                        aac_list.append(aa_type.split('_')[0])
                        amino_acid_count[aa_type] = int(
                            amino_acid_count[aa_type]) + 1
        for aa_count in sorted(amino_acid_count.keys()):
            if not float(amino_acid_count[aa_count]) == 0:
                aac_values.append(float(amino_acid_count[aa_count])
                *100/int(length))
            else:
                aac_values.append(0)
        return ''.join(aac_list), aac_values
    
    def _split_by_n( self, line, n ):
        """A generator to divide a sequence into chunks of n units."""
        seq = [line[i:i+n] for i in range(0, len(line), n)]
        return seq 
        
    def _compute_dp_comp(self, aa_list):
        '''Computes dipeptide composition'''
        aa_count = {}
        aa_comb_set = set()
        aa_final_list = []
        amino_acid_list = list(self._split_by_n(aa_list, 2))
        for comb in combinations(self._amino_acid_list, 2):
            aa_comb_set.add(''.join([comb[0], comb[1]]))
            aa_comb_set.add(''.join([comb[1], comb[0]]))
        for each_aa in amino_acid_list:
            aa_count.setdefault(each_aa, []).append(each_aa)
        for aa in sorted(list(aa_comb_set)):
            try:
                if aa_count[aa]:
                    aa_final_list.append(float(len(aa_count[aa])
                    *100/(len(aa_count.keys()))))
            except KeyError:
                aa_final_list.append(0)
        return aa_final_list

    def _compute_tp_comp(self, aa_list):
        ''''''     
        aa_count = {}
        aa_comb_set = set()
        aa_final_list = []
        amino_acid_list = list(self._split_by_n(aa_list, 3))
        for comb in combinations(self._amino_acid_list, 3):
            aa_comb_set.add(''.join([comb[0], comb[2], comb[1]]))
            aa_comb_set.add(''.join([comb[0], comb[1], comb[2]]))
            aa_comb_set.add(''.join([comb[1], comb[2], comb[0]]))
            aa_comb_set.add(''.join([comb[1], comb[0], comb[2]]))
            aa_comb_set.add(''.join([comb[2], comb[0], comb[1]]))
            aa_comb_set.add(''.join([comb[2], comb[1], comb[0]]))
        for each_aa in amino_acid_list:
            aa_count.setdefault(each_aa, []).append(each_aa)
        for aa in sorted(list(aa_comb_set)):
            try:
                if aa_count[aa]:
                    aa_final_list.append(float(len(aa_count[aa])
                    *100/(len(aa_count.keys()))))
            except KeyError:
                aa_final_list.append(0)
        return aa_final_list
    
class FilteredData(object):
    '''all the xml data detail'''
    def __init__(self, row):
        self.uid = row[0]
        self.entry_name = row[1]
        self.protein_name = row[2]
        self.species = row[3]
        self.length = row[4]
        self.gene_names = row[5]
        self.locus_tag = row[6]
        self.Type = row[7]
        self.go = row[8]
        self.embl = row[9]
        self.pdb = row[10]
        self.kegg = row[11]
        self.interpro = row[12]
        self.pfam = row[13]
        self.pubmed = row[14]
        self.resource = row[15]
        self.resource_id = row[16]
        self.domain_id = row[17]
        self.short_name = row[18]
        self.full_name = row[19]
        self.domain_kw = row[20]
        self.domain_go = row[21]
        self.members = row[22]
        self.dom_length = row[23]
        self.start = row[24]
        self.stop = row[25]
        self.evalue = row[26]
        self.bit_score = row[27]
        self.bits = row[28]
        self.cover_length = row[29]
        self.coverage_percent = row[30]
        self.identity = row[31]
        self.identity_percent = row[32]
        self.similarity = row[33]
        self.similarity_percent = row[34]
        self.gaps = row[35]
        self.gap_percent = row[36]
        self.parameter_tag = row[-1]



#!/usr/bin/env python
# Description = Visualizes different output data from APRICOT analysis

from collections import defaultdict
import os
import sys

try:
    import subprocess
except ImportError:
    print('Python package subprocess is missing. Please install/update.\n')
    sys.exit(0)
try:
    import shutil
except ImportError:
    print('Python package shutil is missing. Please install/update.\n')
    sys.exit(0)


class BiojsVizOfApricotAnalysis(object):
    def __init__(self, annotation_scoring_data,
                 additional_annotation,
                 outpath):
        self._annotation_scoring_data = annotation_scoring_data
        self._additional_annotation = additional_annotation
        self._outpath = outpath
        self._sec_str = self._outpath+'/secondary_structure'
        self._dom_highlight = self._outpath+'/domain_highlighting'
        self._pdb_msa = self._outpath+'/homologous_pdb_msa'
        self._overview = self._outpath+'/overview_and_statistics'
        self._localize = self._outpath+'/subcellular_localization'
        self._annotation_data = []
        self._highlight_dict = {}
        self._uid_key_dict = {}
        self._dom_rank = {}
        self._fasta_dict = {}
        self._secstr_dict = {}
        self._dom_annotation = {}
        self._location_dict = defaultdict(lambda: defaultdict(lambda: []))
        self._sec_str_color = {'H': '#FF6666', 'E': '#33CCCC', 'C': '#FFFFCC'}
        self._localization_dict = defaultdict(
            lambda: defaultdict(lambda: float))
        self._color_list = (
            "Blue", "Green", "Teal", "Lime", "SeaGreen", "MediumTurquoise",
            "Pink", "DarkOliveGreen", "Indigo", "Orange", "SlateBlue",
            "LawnGreen", "Brown", "LightSkyBlue", "LightGreen", "DarkOrchid",
            "GoldenRod", "MidnightBlue", "LightPink", "Gold")

    def viz_all_the_visualization_files(self):
        self.parse_annotation_scoring()
        self.viz_domain_highlights()
        self.domain_highlight_script()
        self.viz_annotation_scoring()
        self.viz_secondary_structure()
        self.sec_str_script()
        self.viz_subcellular_localization()
        self.viz_homologous_pdb_msa()
        
    def parse_annotation_scoring(self):
        with open(self._annotation_scoring_data, 'r') as in_fh:
            for entry in in_fh:
                self._annotation_data.append(entry.strip())
        return self._annotation_data
    
    def viz_domain_highlights(self):
        for entry in self._annotation_data:
            if not entry.startswith('Entry'):
                anno_score = AnnotationScoringColumns(
                    entry.strip().split('\t'))
                prot_name = anno_score.entry_name
                prot_end = int(anno_score.length)-1
                prot_key = '\n'.join(
                    ["\tstart: 0,", "\tend: %s,"
                     % prot_end, '\tname: "%s",' % prot_name,
                     '\thref: "http://www.uniprot.org/uniprot/%s"'
                     % anno_score.uid])
                self._uid_key_dict[anno_score.uid] = prot_key
                self._location_dict[
                    anno_score.uid][anno_score.domain_id].append(
                        '\t{start: %s, end: %s}' % (
                            anno_score.start, anno_score.stop))
                self._dom_annotation[
                    anno_score.domain_id] = anno_score.full_name
                src = anno_score.resource
                if src == 'CDD':
                    self._dom_rank.setdefault(
                        anno_score.uid+':CDD', []).append(
                        anno_score.domain_id)
                    self._highlight_dict.setdefault(
                        prot_key, []).append('\n'.join(
                            ['\t\tstart: %s,' % anno_score.start,
                             '\t\tend: %s,' % anno_score.stop,
                             '\t\tdomain: {', '\t\t\tname: "%s",'
                             % anno_score.domain_id,
                             '\t\t\tid: %s,' % len(
                                 self._dom_rank[anno_score.uid+':CDD']),
                             '\t\t\tdescription: "%s"},' %
                             anno_score.short_name,
                             '\t\tsource: {', '\t\t\tname: "CDD",',
                             '\t\t\thref: null,', '\t\t\tid: 1}']))
                else:
                    self._dom_rank.setdefault(
                        anno_score.uid+':IPR', []).append(
                        anno_score.domain_id)
                    self._highlight_dict.setdefault(
                        prot_key, []).append('\n'.join(
                            ['start: %s,' % anno_score.start,
                             'end: %s,' % anno_score.stop,
                             'domain: {', '\t\tname: "%s",' %
                             anno_score.domain_id,
                             '\t\tid: %s,' % len(
                                 self._dom_rank[anno_score.uid+':IPR']),
                             '\t\tdescription: "%s"},' % anno_score.short_name,
                             'source: {', '\t\tname: "InterPro",',
                             '\t\thref: null,', '\t\tid: 2}']))
        return(self._highlight_dict, self._uid_key_dict, self._location_dict, self._dom_annotation)
                
    def domain_highlight_script(self):
        for uid in self._uid_key_dict.keys():
            header = '\n'.join(['<meta charset="UTF-8">'
            '<link type="text/css" rel="stylesheet" href="http://parce.li/bundle/biojs-vis-protein-viewer@0.1.4">',
            '<script src="https://wzrd.in/bundle/biojs-vis-protein-viewer@0.1.4"></script>',
            '<div id="j-main">', '</div>', '<script>',
            'var ProteinViewer = require("biojs-vis-protein-viewer");'])
            body = '\n'.join(['var highlightData = [', '\t{',
                '\n\t},\n\t{\n'.join(self._highlight_dict[
                    self._uid_key_dict[uid]]), '\t}', '];'])
            panel = '\n'.join(['var highlightLocusData = {',
                               self._uid_key_dict[uid], '};'])
            footer = '\n'.join([
                'var pv = new ProteinViewer({',
                '\tel: document.getElementById("j-main"),',
                '\tdata: highlightData,',
                '\tlocusData: highlightLocusData', '});',
                'pv.render();', '</script>'])
            with open(self._dom_highlight+'/%s.html' % uid, 'w') as out_fh:
                out_fh.write('\n'.join([header, body, panel, footer]))
                
                
    def viz_secondary_structure(self):
        for uid in self._uid_key_dict.keys():
            if uid+'.horiz' in os.listdir(
            self._additional_annotation+'/protein_secondary_structure/'):
                files = uid+'.horiz'
            elif uid+'.plain' in os.listdir(
                self._additional_annotation+'/protein_secondary_structure/'):
                files = uid+'.plain'
                print("\nRaptorX secondary structure files are unavailable.")
                print("Visualizing secondary structure using literature based analysis.\n")
            else:
                print("\nRaptorX/literature-based secondary structure files are unavailable.")
                print("Exiting the current analysis.")
                print("Please re-run the secondary structure prediction by RaptorX\n")
                sys.exit()
            secstr_list = []
            uid_secstr_dict = {}
            sec_data_sites = []
            with open(self._additional_annotation+
                '/protein_secondary_structure/'+files, 'r') as in_fh:
                for entry in in_fh:
                    if 'AA: ' in entry:
                        self._fasta_dict.setdefault(uid,
                        []).append(entry.strip().split('AA: ')[1])
                    if 'Pred: ' in entry:
                        try:
                            secstr_list.append(entry.strip().split('Pred: ')[1])
                        except IndexError:
                            print("\nRaptorX output file is incomplete. Exiting the current analysis.")
                            print("Please re-run the secondary structure prediction by RaptorX\n")
                            sys.exit()
            for i, pred_data in enumerate(''.join(secstr_list)):
                    uid_secstr_dict[i] = pred_data
            for j in range(len(uid_secstr_dict)-1):
                if j == 0:
                    sec_data_sites.append(j)
                if not uid_secstr_dict[j] == uid_secstr_dict[j+1]:
                    sec_data_sites.append(j+1)
                    self._secstr_dict.setdefault(uid, []).append(
                        'mySequence.addHighlight({start:%s, end:%s, color:"Black", background:"%s"});'
                        %(int(sec_data_sites[-2])+1, int(j)+1,
                          self._sec_str_color[uid_secstr_dict[j]]))
            self._secstr_dict.setdefault(uid, []).append(
                'mySequence.addHighlight({start:%s, end:%s, color:"Black", background:"%s"});'
                %(int(sec_data_sites[-1])+1, int(list(uid_secstr_dict.keys())[-1])+1,
                  self._sec_str_color[uid_secstr_dict[j]]))
        return self._fasta_dict, self._secstr_dict
                
    def sec_str_script(self):
        for uid in self._fasta_dict.keys():
            header = '\n'.join(['<meta charset="UTF-8">',
            '<script src="http://code.jquery.com/jquery-1.11.0.min.js"></script>',
            '<script src="https://wzrd.in/bundle/biojs-vis-sequence@0.1.7"></script>',
            '<script src="https://wzrd.in/bundle/biojs-io-fasta@latest"></script>',
            '<div id="snippetDiv"></div>', '<script>',
            'var yourDiv = document.getElementById("snippetDiv");',
            'var Seq = require("biojs-vis-sequence");'])
            footer = '\n'.join([
            'mySequence.on("all",function(name,data){var obj = {name: name, data: data};if(inIframe()){ parent.postMessage(obj, "*") }})',
            'mySequence.onAll(function(name,data){',
            'console.log(arguments);', '});', '};',
            'function inIframe(){try{return window.self!==window.top}catch(e){return true}}', 
            '</script>'])
            body1 = '\n'.join(['var theSequence = "%s";' %
            ''.join(self._fasta_dict[uid]), 'yourDiv.textContent = "";',
            'window.onload = function() {', 'var mySequence = new Seq({',
            '\tsequence : theSequence,', '\ttarget : yourDiv.id,',
            '\tformat : "CODATA",', '\tformatOptions : {',
            '\ttitle:false,', '\tfooter:false', '\t},', '\tid : "%s"' % uid, '});'])
            body2 = '\n'.join(self._secstr_dict[uid])
            dom_list = sorted(list(self._location_dict[uid].keys()))
            annotation_list = []
            for dom_id in dom_list:
                dom_idx = dom_list.index(dom_id)
                annotation_list.append('\n'.join([
                    'mySequence.addAnnotation({', 'name:"Domain-%s",' % str(int(dom_idx)+1),
                    'html:"<br>%s<br>%s</b>",' % (dom_id,
                    self._dom_annotation[dom_id]), 'color:"%s",' % self._color_list[dom_idx],
                    'regions: [', ',\n'.join(self._location_dict[uid][dom_id]), ']});']))
            with open(self._sec_str+'/'+uid+'.html', 'w') as out_fh:
                out_fh.write('\n'.join([header, body1, '\n'.join(annotation_list),
                                        body2, footer]))
        
    def viz_subcellular_localization(self):
        ''''''
        if 'psortb_data_summary.csv' in os.listdir(
            self._additional_annotation+'/protein_localization'):
            total_loc = set()
            with open(
            self._additional_annotation+'/protein_localization/psortb_data_summary.csv',
            'r') as in_fh:
                for entry in in_fh:
                    if not 'Localization' in entry:
                        protein = entry.strip().split('\t')[0]
                        localization = entry.strip().split('\t')[1]
                        if not localization.lower() == 'unknown':
                            score = float(entry.strip().split('\t')[2])
                            self._localization_dict[protein][localization] = score
                            total_loc.add(localization)
            with open(self._localize+'/localization_table.csv', 'w') as out_fh:
                out_fh.write('Proteins\t%s\n' % '\t'.join(sorted(list(total_loc))))
                for each_prot in self._localization_dict.keys():
                    for localization in self._localization_dict[each_prot]:
                        entry_list = list('0'*len(total_loc))
                        loc_idx = sorted(list(total_loc)).index(localization)
                        entry_list[loc_idx] = self._localization_dict[each_prot][localization]
                        out_fh.write("%s\t%s\n" % (each_prot, '\t'.join(map(str, entry_list))))
            self._create_localization_heatmap()
        else:
            print("\nPsortB-based localization prediction files are unavailable.")
            print("Exiting the current analysis.")
            print("Please re-run the localization prediction by PsortB\n")
            sys.exit()
        
    def _create_localization_heatmap(self):
        ''''''
        plot_file = self._localize+'/localization_heatmap.pdf'
        infile = self._localize+'/localization_table.csv'
        with open(self._localize+'/localization_heatmap.R', 'w') as r_fh:
            r_fh.write('\n'.join(['library(gplots)', 'library(RColorBrewer)', 'display.brewer.all()',
        'data <- read.csv("%s", header=T, sep = "\\t")' % infile,
        'rnames <- data[,1]',  'data_matrix <- data.matrix(data[,2:ncol(data)])',
        'data_matrix[is.na(data_matrix)] <- 0', 'data_matrix[is.nan(data_matrix)] <- 0',
        'data_matrix[is.infinite(data_matrix)] <- max(data_matrix)',
        'rownames(data_matrix) <- rnames', 'pdf(file="%s")' % plot_file,
        'out_map <- heatmap.2(data_matrix, dendrogram = "none", Rowv = FALSE, \
        Colv = FALSE, col=brewer.pal(9,"YlGn"), margins=c(5,8), \
        cexCol=0.8, cexRow=0.8, key.title="PsortB Pred-value", key.xlab="", key.ylab="")',
        'dev.off()']))
        subprocess.Popen(['Rscript %s/localization_heatmap.R' %
                          self._localize], shell=True).wait()
        
    def viz_homologous_pdb_msa(self):
        header = '\n'.join(['<meta charset="UTF-8">',
        '<link type="text/css" rel="stylesheet" href="http://parce.li/bundle/msa@0.4.8">',
        '<script src="https://wzrd.in/bundle/msa@0.4.8"></script>',
        '<script src="https://wzrd.in/bundle/biojs-io-fasta@latest"></script>',
        '<script src="https://wzrd.in/bundle/biojs-io-clustal@latest"></script>',
        '<script src="https://wzrd.in/bundle/biojs-io-gff@latest"></script>',
        '<script src="https://wzrd.in/bundle/xhr@latest"></script>',
        '<div id="snippetDiv"></div>', '<script>',
        'var rootDiv = document.getElementById("snippetDiv");',
        'var msa = require("msa");', 'var menuDiv = document.createElement("div");',
        'var msaDiv = document.createElement("div");',
        'rootDiv.appendChild(menuDiv);', 'rootDiv.appendChild(msaDiv);'])
        footer = '\n'.join(['opts.conf = {', '\tdropImport: true,',
        '\tmanualRendering: true', '};', 'opts.vis = {', '\tconserv: false,',
        '\toverviewbox: false,', '\tseqlogo: true,', '\tmetacell: true', '};',
        'opts.zoomer = {', '\tlabelIdLength: 20', '};', 'var m = msa(opts);',
        'gg = m;', 'm.u.file.importURL(url, function() {',
        '\tvar defMenu = new msa.menu.defaultmenu({', '\t\tel: menuDiv,',
        '\t\tmsa: m', '\t});', '\tdefMenu.render();', '\tm.render();', '});',
        'm.g.on("all",function(name,data){var obj = {name: name, data: data};if(inIframe()){ parent.postMessage(obj, "*") }})',
        'function inIframe(){try{return window.self!==window.top}catch(e){return true}}',
        '</script>'])
        body = '//EDIT PATH\n'.join([
        'var url = "https://github.com/malvikasharan/APRICOT/blob/master/Biojs_dependencies/data/biojs_msa_tab.clustal";'
        'var opts = {', '\tel: msaDiv', '};'])
        with open(self._pdb_msa+'/Biojs_pdb_msa_tab.html', 'w') as out_fh:
            out_fh.write('\n'.join([header, body, footer]))
            
        for files in os.listdir(self._additional_annotation+'/pdb_sequence_prediction/'):
            if '_top5.fasta' in files:
                shutil.copyfile(
                self._additional_annotation+'/pdb_sequence_prediction/'+files,
                self._pdb_msa+'/'+files)
                subprocess.Popen(['bin/reference_db_files/clustal/clustalw2 %s' %
                    self._pdb_msa+'/'+files], shell=True).wait()
                
        print("\nPlease open the BioJS MSA tab generated in Biojs_pdb_msa_tab.html.")
        print("Import MSA files (.aln) in the BioJS MSA tab to visualize the alignment.\n")


class AnnotationScoringColumns(object):
    '''Column information of annotation scoring file'''
    def __init__(self, row):
        self.uid = row[0]
        self.entry_name = row[1]
        self.prot_name = row[2]
        self.species = row[3]
        self.length = row[4]
        self.resource = row[5]
        self.resource_id = row[6]
        self.domain_id = row[7]
        self.short_name = row[8]
        self.full_name = row[9]
        self.domain_length = row[10]
        self.start = row[11]
        self.stop = row[12]
        self.ref_seq = row[13]
        self.q_seq = row[14]
        self.ref_ss = row[15]
        self.q_ss = row[16]
        self.mol_mass = row[17]
        self.iso_pt = row[18]
        self.solub = row[19]
        self.vdw = row[20]
        self.coverage = row[21]
        self.cov_by_dom = row[22]
        self.identity = row[23]
        self.iden_by_cov = row[24]
        self.similarity = row[25]
        self.sim_by_cov = row[26]
        self.gap = row[27]
        self.gap_by_cov = row[28]
        self.AA_RO = row[29]
        self.SS_RO = row[30]
        self.PC_RO = row[31]
        self.AAC_ED = row[32]
        self.PCC_ED = row[33]
        self.DPC_ED = row[34]
        self.TPC_ED = row[35]
        



#!/usr/bin/env python
# Description = Filters prteins that contains domains of interest

import os


class FilterPredictedDomains(object):
    '''Filter the predicted domain by
    parameters and domain of interest'''
    def __init__(self, prediction_method,
                 analysis_result_path,
                 domain_description_file,
                 go_path,
                 filtered_result_path,
                 all_prediction_output,
                 filter_parameters):
        self._prediction_method = prediction_method
        self._analysis_result_path = analysis_result_path
        self._domain_description_file = domain_description_file
        self._go_path = go_path
        self._filtered_result_path = filtered_result_path
        self._all_prediction_output = all_prediction_output
        self._filter_parameters = filter_parameters
        
        self._id_description_dict = {}
        self._domain_length = {}
        self._cdd_go_dict = {}
        self._interpro_go_dict = {}
        self._filter_parameters_dict = {}
        self._result_detail_dict = {}
        
    def streamline_filter_predicted_domains(self):
        '''To call from apricot'''
        self.read_id_description_file()
        self.read_filter_parameters()
        self.summarize_analysis_result_files()
        self.create_filtered_result_file()
        
    def read_id_description_file(self):
        '''read id file.'''
        with open(self._domain_description_file,
                  'r') as id_description_fh:
            for entry in id_description_fh:
                if 'ReferenceId' not in entry:
                    try:
                        length = entry.split('\t')[-2]
                        parent_id = entry.split('\t')[1]
                        if 'smart' in parent_id:
                            parent_id = parent_id.replace('smart', 'SM')
                        elif 'pfam' in parent_id:
                            parent_id = parent_id.replace('pfam', 'PF')
                        if parent_id not in set(self._domain_length.keys()):
                            self._domain_length[parent_id] = length
                        self._id_description_dict[entry.split(
                            '\t')[0]] = entry.strip().replace(
                            entry.split('\t')[1], parent_id)
                        self._id_description_dict[entry.split(
                            '\t')[1]] = entry.strip().replace(
                            entry.split('\t')[1], parent_id)
                    except IndexError:
                        pass
        return self._id_description_dict, self._domain_length
    
    def read_filter_parameters(self):
        '''Records the filtering parameter for cut-off'''
        for param in self._filter_parameters:
            param_key = param.split(':')[0]
            param_val = param.split(':')[1]    
            if not param_val == 'NA':
                self._filter_parameters_dict[param_key] = float(param_val)
        return self._filter_parameters_dict
    
    def summarize_analysis_result_files(self):
        '''summarize all the the results in dictionary'''
        if self._prediction_method == 'cdd':
            self.read_cdd_go_file()
            self.summarize_rps_analysis_result_files()
        elif self._prediction_method == 'ipr':
            self.read_interpro_go_file()
            self.summarize_ipr_analysis_result_files()
            
    def read_cdd_go_file(self):
        '''read mapped file for cdd id to go.'''
        with open(self._go_path+'/mapped_cdd_to_go.csv',
                  'r') as cdd_go_fh:
            for cdd_entry in cdd_go_fh:
                self._cdd_go_dict[cdd_entry.split(
                    '\t')[0]] = cdd_entry.split('\t')[-1].strip()
        return self._cdd_go_dict
        
    def read_interpro_go_file(self):
        '''read mapped file for interpro id to go.'''
        with open(self._go_path+'/mapped_interpro_to_go.csv',
                  'r') as interpro_go_fh:
            for interpro_entry in interpro_go_fh:
                self._interpro_go_dict[interpro_entry.split(
                    '\t')[0]] = interpro_entry.split('\t')[-1].strip()
        return self._interpro_go_dict
        
    def summarize_rps_analysis_result_files(self):
        '''summarize all the RPS-BLAST results in dictionary'''
        for individual_rps_result_file in os.listdir(
                self._analysis_result_path):
            protein_id = individual_rps_result_file.split('.')[0]
            with open(
                    self._analysis_result_path+'/'+individual_rps_result_file
            ) as individual_rps_result_fh:
                for individual_rps_result_section in \
                individual_rps_result_fh.read().split('>gnl'):
                    if individual_rps_result_section.startswith('|CDD|'):
                        stat_data = individual_rps_result_section.split(
                            "Score = ")
                        for individual_rps_result in stat_data[0].split('\n'):
                            if individual_rps_result.startswith('|CDD|'):
                                cdd_main = self._compile_cdd_main(
                                    individual_rps_result)
                            if 'Length = ' in individual_rps_result:
                                try:
                                    length = individual_rps_result.split(
                                        'Length = ')[1].strip()
                                except:
                                    length = self._domain_length[
                                        cdd_main.split('\t')[1]]
                        for each_stat_group in stat_data[1:]:
                            check_list = set()
                            parameter_dict = self._compile_cdd_stat(
                                each_stat_group)
                            coverage_percent = "%.4f" % ((int(
                                parameter_dict[
                                    "cover_length"])/int(length)) * 100)
                            if float(coverage_percent) > 100:
                                coverage_percent = 100
                            similarity_percent = "%.4f" % ((int(
                                parameter_dict["similarity_value"].split(
                                    '/')[0])/int(length))*100)
                            identity_percent = "%.4f" % ((int(
                                parameter_dict["identity_value"].split(
                                    '/')[0])/int(length))*100)
                            if parameter_dict["gaps_value"] == 'None':
                                gap_percent = 0
                            else:
                                gap_percent = "%.4f" % ((int(
                                    parameter_dict["gaps_value"].split(
                                        '/')[0])/int(length)) * 100)
                            evalue = parameter_dict["evalue"]
                            bits = parameter_dict["bit"].split(' ')[0]
                            start = parameter_dict["start"]
                            stop = parameter_dict["stop"]
                            compiled_data = '\t'.join([
                                'CDD', protein_id, cdd_main,
                                str(length), str(start), str(stop),
                                str(evalue),
                                str(parameter_dict["bit"]),
                                str(bits),
                                str(parameter_dict["cover_length"]),
                                str(coverage_percent),
                                str(parameter_dict["identity_value"]),
                                str(identity_percent),
                                str(parameter_dict["similarity_value"]),
                                str(similarity_percent),
                                str(parameter_dict["gaps_value"]),
                                str(gap_percent)])
                            parameter_dict["coverage"] = coverage_percent
                            parameter_dict["identity"] = identity_percent
                            parameter_dict["similarity"] = similarity_percent
                            parameter_dict["gaps"] = gap_percent
                            if self._filter_parameters_dict:
                                for param in self._filter_parameters_dict.keys():
                                    if not param == 'evalue':
                                        if not self._filter_parameters_dict[param] == 'NA':
                                            if not float(parameter_dict[param]) >= float(
                                                self._filter_parameters_dict[param]):
                                                check_list.add('no')
                                    else:
                                        if not self._filter_parameters_dict[param] == 'NA':
                                            if not float(parameter_dict[param]) <= float(
                                                self._filter_parameters_dict[param]):
                                                check_list.add('no')
                                if 'no' not in check_list:
                                    self._result_detail_dict.setdefault(
                                        cdd_main.split('\t')[0], []).append(
                                            compiled_data+'\tParameterSelected')
                                    self._result_detail_dict.setdefault(
                                        cdd_main.split('\t')[1], []).append(
                                            compiled_data+'\tParameterSelected')
                                else:
                                    self._result_detail_dict.setdefault(
                                        cdd_main.split('\t')[0], []).append(
                                            compiled_data+'\tParameterDiscarded')
                                    self._result_detail_dict.setdefault(
                                        cdd_main.split('\t')[1], []).append(
                                        compiled_data+'\tParameterDiscarded')
        return self._result_detail_dict
    
    def _compile_cdd_main(self, individual_rps_result):
        '''Collects non statistical information
        from CDD derived predictions'''
        cdd_main = ''
        for entry in individual_rps_result.split('\n'):
            if '|CDD|' in entry:
                pssm_id = entry\
                          .split('|CDD|')[1].split(' ')[0]
                parent_id = entry\
                               .split(',')[0].split(' ')[1]
                if 'smart' in parent_id:
                    parent_id = parent_id.replace('smart', 'SM')
                elif 'pfam' in parent_id:
                    parent_id = parent_id.replace('pfam', 'PF')
                try:
                    go_id = self._cdd_go_dict[pssm_id]
                except:
                    go_id = "Not-known"
                if pssm_id in self._id_description_dict.keys():
                    short_name = self._id_description_dict[
                        pssm_id].split('\t')[2]
                    pssm_detail = self._id_description_dict[
                        pssm_id].split('\t')[3]
                    members = self._id_description_dict[
                        pssm_id].split('\t')[-3]
                    keyword = self._id_description_dict[
                        pssm_id].strip().split('\t')[-1]
                elif parent_id in self._id_description_dict.keys():
                    short_name = self._id_description_dict[
                        parent_id].split('\t')[2]
                    pssm_detail = self._id_description_dict[
                        parent_id].split('\t')[3]
                    members = self._id_description_dict[
                        parent_id].split('\t')[-3]
                    keyword = self._id_description_dict[
                        parent_id].strip().split('\t')[-1]
                else:
                    try:
                        short_name = entry.split(',')[2]
                    except IndexError:
                        short_name = entry[2]
                    if ";" in short_name:
                        short_name = short_name.split(';')[0]
                    elif "." in short_name:
                        short_name = short_name.split('.')[0]
                    pssm_detail = short_name
                    members = 'NotSelected'
                    keyword = 'NA'
                cdd_main = (
                '%s\t%s\t%s\t%s\t%s\t%s\t%s' %
                (pssm_id, parent_id, short_name, pssm_detail, 
                 keyword, go_id, members))
            return cdd_main

    def _compile_cdd_stat(self, each_stat_group):
        '''Collects statistical information
        from CDD derived predictions'''
        parameter_dict = {}
        start_list = []
        stop_list = []
        for each_stat_data in each_stat_group.split('\n'):
            if 'Expect' in each_stat_data:
                score = each_stat_data.split(',')[0]
                expect = each_stat_data.split(
                    'Expect = ')[1].strip()
                if expect.startswith('e'):
                    expect = float(expect.replace('e', '1e'))
                parameter_dict["evalue"] = float(expect)
                parameter_dict["bit"] = score
            if 'Identities = ' in each_stat_data:
                identities = each_stat_data.split(
                    'Identities = ')[1].split(',')[0]
                parameter_dict["identity_value"] = identities
                try:
                    gaps = each_stat_data.split(
                        'Gaps = ')[1].strip()
                    parameter_dict["gaps_value"] = gaps
                    positives = each_stat_data.split(
                        'Positives = ')[1].split(',')[0]
                    parameter_dict["similarity_value"] = positives
                except:
                    gaps = 'None'
                    parameter_dict["gaps_value"] = gaps
                    positives = each_stat_data.split(
                        'Positives = ')[1].strip()
                    parameter_dict["similarity_value"] = positives
            if 'Query:' in each_stat_data:
                start_list.append(each_stat_data.split(' ')[1])
                stop_list.append(each_stat_data.split(' ')[-1].strip())
        start = start_list[0]
        stop = stop_list[-1]
        parameter_dict["start"] = start
        parameter_dict["stop"] = stop
        parameter_dict["cover_length"] = int(stop)-int(start)
        return parameter_dict
    
    def summarize_ipr_analysis_result_files(self):
        '''summarize all the InterProScan results in dictionary'''
        for individual_ipr_result_file in os.listdir(
            self._analysis_result_path):
            if '.tsv' in individual_ipr_result_file:
                protein_id = individual_ipr_result_file.split('.')[0]
                if '_' in protein_id:
                    protein_id = protein_id.split('_')[0]
                with open(
                self._analysis_result_path+'/'+individual_ipr_result_file,
                    'r') as individual_ipr_result_fh:
                    for ipr_entry in individual_ipr_result_fh:
                        try:
                            parameter_tag = 'ParameterNotApplicable'
                            parent_db = ipr_entry.split('\t')[3]
                            parent_db_id = ipr_entry.split('\t')[4]
                            name = ipr_entry.split('\t')[5]
                            start = ipr_entry.split('\t')[6]
                            stop = ipr_entry.split('\t')[7]
                            expect = ipr_entry.split('\t')[8]
                            if expect == '-':
                                expect = 0
                            if self._filter_parameters_dict:
                                if 'evalue' in self._filter_parameters_dict.keys() and not self._filter_parameters_dict['evalue'] == 'NA':
                                    if float(expect) <= float(
                                        self._filter_parameters_dict['evalue']):
                                        parameter_tag = 'ParameterSelected'
                                    else:
                                        parameter_tag = 'ParameterDiscarded'
                            try:
                                ipr_id = ipr_entry.split('\t')[11]
                            except:
                                ipr_id = ipr_entry.split('\t')[4]
                            try:
                                go_id = '%s' % (
                                    self._interpro_go_dict[ipr_id])
                            except:
                                go_id = 'NA'
                            if self._id_description_dict.get(
                                parent_db_id):
                                long_name = self._id_description_dict[
                                    parent_db_id].split('\t')[3]
                                members = ('%s' %
                                           self._id_description_dict[
                                    parent_db_id].split('\t')[-3])
                                keyword = ('%s' %
                                           self._id_description_dict[
                                    parent_db_id].strip().split('\t')[-1])
                            else:
                                long_name = name
                                members = 'NotSelected'
                                keyword = 'NA'
                            try:
                                length = self._domain_length[parent_db_id]
                            except KeyError:
                                length = '-'
                            cover_length = int(stop)-int(start)
                            if not length == '-' and not length == 'NA':
                                cover_percent = "%.4f" % ((int(cover_length)/int(length))*100)
                            else:
                                cover_percent = '-'
                            ipr_data = ('\t'.join(['INTERPRO',
                            protein_id, ipr_id, parent_db_id, name,
                            long_name, keyword, go_id, members, str(length),
                            str(start), str(stop), str(expect), '\t'.join('-'*2),
                            str(cover_length), str(cover_percent),
                            '\t'.join('-'*6), parameter_tag]))  
                            self._result_detail_dict.setdefault(
                                ipr_id, set()).add(ipr_data)
                        except IndexError:
                            pass
        return self._result_detail_dict
    
    def create_filtered_result_file(self):
        '''select the result data with domain of interest'''
        self._candidate_set = set()
        self._discard_set = set()
        self._id_set = set()
        header = "\t".join([
            'Resource', 'UniProtID', 'ResourceID',
            'DomainID', 'ShortName', 'FullName', 'DomainKeyword',
            'DomainGo', 'Members', 'DomainLength', 'Start', 'Stop',
            'Evalue', 'BitScore', 'Bits', 'CoverLength',
            'CoveragePercent', 'Identity', 'IdentityPercent',
            'Similarity', 'SimilarityPercent', 'Gaps', 'GapPercent',
            'ParameterFilterTag'])
        summary_file = open(
            self._all_prediction_output + '/%s_unfiltered_all_prediction.csv' %
            self._prediction_method, 'w')
        filtered_file = open(
            self._filtered_result_path + '/' +
            self._prediction_method+'_filtered.csv', 'w')
        summary_file.write(header+'\tDomainFilterTag\n')
        filtered_file.write(header+'\n')
        id_file = open(
            self._filtered_result_path + '/' +
            self._prediction_method+'_filtered_id.csv', 'w')
        for selected_id in set(self._id_description_dict.keys()):
            if selected_id in set(self._result_detail_dict.keys()):
                for each_data in self._result_detail_dict[selected_id]:
                    self._candidate_set.add(each_data)
        for each_candidate in list(self._candidate_set):
            filtered_file.write('%s\n' % each_candidate)
            summary_file.write('%s\tSelected\n' % each_candidate)
        print("Filtered uniprot IDs by %s:" % self._prediction_method)
        for candidate_entry in self._candidate_set:
            uid = candidate_entry.split('\t')[1]
            db_id = candidate_entry.split('\t')[2]
            self._id_set.add('%s\t%s' % (uid, db_id))
        id_list = sorted(self._id_set)
        print('UID\tDomainID\n'+'\n'.join(id_list))
        id_file.write('\n'.join(id_list))
        for remaining_id in set(self._result_detail_dict.keys()):
            for each_entry in self._result_detail_dict[remaining_id]:
                if each_entry not in self._candidate_set:
                    self._discard_set.add(each_entry)
        for each_discard in list(self._discard_set):
            summary_file.write('%s\tNotSelected\n' % each_discard)
        summary_file.close()
        filtered_file.close()
        id_file.close()



#!/usr/bin/env python
# Description = Checks and stores the most recent version information


from datetime import datetime as DT
import sys

input_file = sys.argv[1]
output_file = sys.argv[2]

month_dict = {'Jan': 1, 'Feb': 2, 'Mar': 3, 'Apr': 4, 'May': 5,
              'Jun': 6, 'Jul': 7, 'Aug': 8, 'Sep': 9, 'Oct': 10,
              'Nov': 11, 'Dec': 12}
with open(input_file) as in_fh:
    version_link = {}
    version_list = []
    for entry in in_fh.read().split('<pre>')[1].split('\n'):
        if 'Directory' in entry and not 'lookup_service' \
           in entry and not 'users' in entry and not 'old_releases' in entry:
            date_data = entry.split('Directory')[0].strip().replace(
                ':', ' ').split(' ')
            year = int(date_data[0])
            month = int(month_dict[date_data[1]])
            date = int(date_data[2])
            try:
                time = int(date_data[3])
            except IndexError:
                time = 0
            version = entry.split('/</a>')[0].split('">')[-1]
            version_link[DT(year, month, date, time)] = version
            version_list.append(DT(year, month, date, time))
    latest_version = max(version_list)
    print("Current version: %s" % version_link[latest_version])
    with open(output_file, 'w') as out_fh:
        out_fh.write(version_link[latest_version])



#!/usr/bin/env python
# Description = selects domains from interpro database


class RnaRelatedDomainSelection(object):
    '''classification of data'''
    def __init__(self, keywords_file,
                 ipr_whole_data_file,
                 interpro_mapped_cdd,
                 domain_data_path):
        self._keywords_file = keywords_file
        self._ipr_whole_data_file = ipr_whole_data_file
        self._interpro_mapped_cdd = interpro_mapped_cdd
        self._domain_data_path = domain_data_path
        
        self.ipr_whole_data_list = []
        self._ipr_dict = {}
        self._mapped_interpro_length = {}
        
    def read_keyword_file(self):
        '''reads user provided keywords for domain selection'''
        self.keyword_list = [rna_keyword.strip()
                             for rna_keyword in open(
                                self._keywords_file)]
        return self.keyword_list
    
    def read_interpro_mapped_cdd_file(self):
        '''Parses cdd interpro mapped file for
        the extraction of common information'''
        with open(self._interpro_mapped_cdd, 'r') as in_fh:
            for entry in in_fh:
                cdd_member = entry.split('\t')[3]
                domain_length = entry.split('\t')[4].strip()
                self._mapped_interpro_length[cdd_member] = domain_length
        self._mapped_interpro_length
        
    def read_ipr_whole_data_file(self):
        '''Parses CDD annotation data from table'''
        with open(self._ipr_whole_data_file, 'r') as in_fh:
            for ipr_id_entry in in_fh:
                self.ipr_whole_data_list.append(ipr_id_entry.strip())
                self._ipr_dict[ipr_id_entry.strip()] = ipr_id_entry.strip(
                    ).split('\t')[2]
        return self.ipr_whole_data_list, self._ipr_dict
        
    def create_keyword_selected_domain_file(self):
        '''Creates a file with CDD derived domain of interest'''
        self._keyword_annotation_dict = {}
        self._keyword_selected_domain = []
        for ipr_entry in self.ipr_whole_data_list:
            for keyword in self.keyword_list:
                if ' ' in keyword:
                    key_list = []
                    for each_key in keyword.split(' '):
                        key_list.append(each_key)
                    match = re.search(r'\b%s*|\W%s\b'%(key_list[0].lower(),
                                                       key_list[1].lower()),
                                      self._ipr_dict[ipr_entry])
                    if match:
                        self._keyword_annotation_dict.setdefault(
                            keyword, []).append(ipr_entry)
                else:
                    match = re.search(r'\b%s\b' % keyword, self._ipr_dict[
                        ipr_entry])
                    if match:
                        self._keyword_annotation_dict.setdefault(
                            keyword, []).append(ipr_entry)
        for fkeyword in self.keyword_list:
            fkeyword = fkeyword.replace(' ', '_')
            with open(self._domain_data_path +'/' +
                      fkeyword+'_related_ipr_ids.tab', 'w') as key_fh:
                if self._keyword_annotation_dict.get(fkeyword):
                    for each_entry in self._keyword_annotation_dict[fkeyword]:
                        each_entry = each_entry.replace(each_entry.split(
                            '\t')[3], " ".join(each_entry.split('\t')[3].split(
                            ))).replace(';', ',')
                        cdd_id = each_entry.split('\t')[1]
                        if cdd_id in set(self._mapped_interpro_length.keys()):
                            length = self._mapped_interpro_length[cdd_id]
                        else:
                            length = 'NA'
                        self._keyword_selected_domain.append("%s\t%s" % (
                            each_entry, length))
                        key_fh.write("%s\t%s\n" % (each_entry, length))
        uniq_keyword_selected_domains = list(
            set(self._keyword_selected_domain))
        with open(self._domain_data_path+'/all_keyword_selected_ipr_data.tab',
                  'w') as keyword_selected_domain_file:
            for domain_entry in uniq_keyword_selected_domains:
                keyword_selected_domain_file.write(
                    '%s\n' % str(domain_entry))



#!/usr/bin/env python
# Description = Prepares query protein data for analysis


def create_uid_query_file(uids, query_path):
    ''''''
    with open(query_path, 'a') as out_fh:
        if ',' in uids or ' ' not in uids:
            for uid in uids.split(','):
                out_fh.write("%s:%s\n" % (uid, uid))
        else:
            print(
                "Query Error Message:\nPlease check that the query UniProt "
                "ids are separated by comma\n")


def create_gene_query_file(geneids, query_path, proteome_path):
    entry_dict = create_gene_dict(proteome_path)
    with open(query_path, 'a') as out_fh:
        if ',' in geneids or ' ' not in geneids:
            mapped_data = map_gene_to_uid(geneids, entry_dict)
            for each_map in sorted(mapped_data.keys()):
                entry = "%s:%s" % (
                    each_map, ','.join(list(mapped_data[each_map])))
                print(entry)
                out_fh.write(entry+'\n')
        else:
            print(
                "Query Error Message:\nPlease check that the query "
                "UniProt ids are separated by comma\n")


def create_gene_dict(proteome_path):
    entry_dict = {}
    with open(proteome_path, 'r') as in_fh:
        for entry in in_fh:
            uid = entry.split('\t')[0]
            entry_dict[entry] = uid
    return entry_dict


def map_gene_to_uid(geneids, entry_dict):
    mapped_data = {}
    if ',' in geneids or ' ' not in geneids:
        for each_gene in geneids.split(','):
            if ' ' in each_gene:
                each_gene = each_gene.replace(' ', '')
            gene = each_gene.lower()
            for entry in entry_dict.keys():
                if gene in entry.split('\t')[0].lower():
                    mapped_data.setdefault(
                        each_gene, set()).add(entry_dict[entry])
                elif gene in entry.split('\t')[1].split('_')[0].lower():
                    mapped_data.setdefault(
                        each_gene, set()).add(entry_dict[entry])
                elif gene+' ' in entry.split('\t')[3].lower(
                ) or ' '+gene in entry.split('\t')[3].lower():
                    mapped_data.setdefault(
                        each_gene, set()).add(entry_dict[entry])
                elif gene+' ' in entry.split('\t')[4].lower(
                ) or ' '+gene in entry.split('\t')[4].lower():
                    mapped_data.setdefault(
                        each_gene, set()).add(entry_dict[entry])
            if gene not in mapped_data.keys():
                mapped_data.setdefault(each_gene, set()).add('unmapped')
    return mapped_data


def create_proteome_query_file(query, query_path, proteome_path):
    ''''''
    with open(query_path, 'a') as out_fh:
        with open(proteome_path, 'r') as in_fh:
            for entry in in_fh:
                if 'Entry' not in entry:
                    out_fh.write("%s\n" % (entry.split('\t')[0]))



#!/usr/bin/env python

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(name='bio-APRICOT',
      version='1.1.4',
      description=('Sequence-based identification and characterization '
                   'of protein classes'),
      author='Malvika Sharan',
      author_email='malvikasharan@gmail.com',
      url='https://www.python.org/pypi/bio-apricot',
      packages=['apricotlib', 'bin'],
      install_requires=[
        "biopython >= 1.66",
        "requests >= 2.10.0",
        "matplotlib >= 1.5.1",
        "openpyxl >= 2.3.1",
        "numpy >= 1.9.0",
        "scipy >= 0.16.0"
      ],
      scripts=['bin/apricot'],
      license='ISC License (ISCL)',
      classifiers=[
          'License :: OSI Approved :: ISC License (ISCL)',
          'Operating System :: POSIX',
          'Programming Language :: Python :: 3',
          'Topic :: Scientific/Engineering :: Bio-Informatics',
      ])
